/*
 * based on: snul.vala (part of Snul)
 *
 * Snul: is a simple library that implements
 *       a tcp socket client in vala
 *
 * Copyright (C) 2008  Andrea Del Signore
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 *
 * Author:
 *     Andrea Del Signore <sejerpz@tin.it>
 */

#ifndef __SNUL_GNIO_H__
#define __SNUL_GNIO_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gnio/gnio.h>

G_BEGIN_DECLS


#define SNUL_TYPE_TCP_CLIENT (snul_tcp_client_get_type ())
#define SNUL_TCP_CLIENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SNUL_TYPE_TCP_CLIENT, SnulTcpClient))
#define SNUL_TCP_CLIENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SNUL_TYPE_TCP_CLIENT, SnulTcpClientClass))
#define SNUL_IS_TCP_CLIENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SNUL_TYPE_TCP_CLIENT))
#define SNUL_IS_TCP_CLIENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SNUL_TYPE_TCP_CLIENT))
#define SNUL_TCP_CLIENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SNUL_TYPE_TCP_CLIENT, SnulTcpClientClass))

typedef struct _SnulTcpClient SnulTcpClient;
typedef struct _SnulTcpClientClass SnulTcpClientClass;
typedef struct _SnulTcpClientPrivate SnulTcpClientPrivate;

typedef enum  {
	SNUL_TCP_SOCKET_ERROR_LOOKUP_SERVICE_FAILED,
	SNUL_TCP_SOCKET_ERROR_LOOKUP_HOSTNAME_FAILED,
	SNUL_TCP_SOCKET_ERROR_SOCKET_CREATION_ERROR,
	SNUL_TCP_SOCKET_ERROR_SOCKET_CONNECT_ERROR,
	SNUL_TCP_SOCKET_ERROR_SOCKET_ALREADY_OPEN,
	SNUL_TCP_SOCKET_ERROR_SOCKET_CLOSED,
	SNUL_TCP_SOCKET_ERROR_SOCKET_BROKEN,
	SNUL_TCP_SOCKET_ERROR_SOCKET_HUP,
	SNUL_TCP_SOCKET_ERROR_IOCHANNEL_ERROR,
	SNUL_TCP_SOCKET_ERROR_IOCHANNEL_NVAL,
	SNUL_TCP_SOCKET_ERROR_OPERATION_TIMEOUT
} SnulTcpSocketError;
#define SNUL_TCP_SOCKET_ERROR snul_tcp_socket_error_quark ()
struct _SnulTcpClient {
	GObject parent_instance;
	SnulTcpClientPrivate * priv;
};

struct _SnulTcpClientClass {
	GObjectClass parent_class;
	void (*on_connected) (SnulTcpClient* self, GSocketConnection* client);
	void (*on_data_received) (SnulTcpClient* self, const char* buffer, gsize len);
	void (*on_disconnected) (SnulTcpClient* self);
	void (*on_error) (SnulTcpClient* self, GError* error);
};


GQuark snul_tcp_socket_error_quark (void);
void snul_tcp_client_connect (SnulTcpClient* self, const char* address, const char* port, GError** error);
gsize snul_tcp_client_send (SnulTcpClient* self, const char* buffer);
void snul_tcp_client_disconnect (SnulTcpClient* self);
void snul_tcp_client_on_connected (SnulTcpClient* self, GSocketConnection* client);
void snul_tcp_client_on_data_received (SnulTcpClient* self, const char* buffer, gsize len);
void snul_tcp_client_on_disconnected (SnulTcpClient* self);
void snul_tcp_client_on_error (SnulTcpClient* self, GError* error);
SnulTcpClient* snul_tcp_client_new (void);
const char* snul_tcp_client_get_address (SnulTcpClient* self);
const char* snul_tcp_client_get_first_address (SnulTcpClient* self);
void snul_tcp_client_set_first_address (SnulTcpClient* self, const char* value);
const char* snul_tcp_client_get_port (SnulTcpClient* self);
GType snul_tcp_client_get_type (void);


G_END_DECLS

#endif
