/*
 *  tuntun-applet.vala is a part of Tuntun
 *
 *  Tuntun: a simple applet to manage OpenVPN connections
 *
 *  Copyright (C) 2008  Andrea Del Signore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
 *
 *  Author:
 *     Andrea Del Signore <sejerpz@tin.it>       
 */

#include "tuntun-applet.h"
#include <libbonoboui.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <libnotify/notify.h>
#include <glib/gi18n-lib.h>
#include <gdk/gdk.h>
#include <config.h>
#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include "tuntun.h"
#include "tuntun-connections-dialog.h"
#include "tuntun-log-window.h"
#include "tuntun-utils.h"
#include "tuntun-constants.h"
#include "tuntun-tooltip.h"
#include "tuntun-connections.h"
#include "tuntun-connection.h"
#include "tuntun-connection-info.h"
#include "tuntun-auth-dialog.h"




struct _TuntunPanelAppletPrivate {
	char* _right_click_menu_xml;
	BonoboUIVerb* _verbs;
	gint _verbs_length1;
	TuntunTuntun* _tuntun;
	TuntunConnectionsDialog* _settings;
	TuntunLogWindow* _log;
	GtkImage* _image;
	GdkPixbuf** _images;
	gint _images_length1;
	gint _image_idx;
	gint _animation_status;
};

#define TUNTUN_PANEL_APPLET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TUNTUN_TYPE_PANEL_APPLET, TuntunPanelAppletPrivate))
enum  {
	TUNTUN_PANEL_APPLET_DUMMY_PROPERTY
};
static void _tuntun_panel_applet_on_connection_status_changed_tuntun_connections_connection_status_changed (TuntunConnections* _sender, TuntunConnection* connection, gpointer self);
static void _tuntun_panel_applet_on_connection_fatal_error_tuntun_connections_connection_fatal_error (TuntunConnections* _sender, TuntunConnection* connection, const char* error, gpointer self);
static void _tuntun_panel_applet_on_connection_authentication_required_tuntun_connections_authentication_required (TuntunConnections* _sender, TuntunConnection* connection, TuntunAuthenticationModes mode, const char* type, gpointer self);
static void _tuntun_panel_applet_on_connection_authentication_failed_tuntun_connections_authentication_failed (TuntunConnections* _sender, TuntunConnection* connection, TuntunAuthenticationModes mode, const char* type, gpointer self);
static void _tuntun_panel_applet_on_connection_activity_tuntun_connections_connection_activity (TuntunConnections* _sender, TuntunConnection* connection, gpointer self);
static void _tuntun_panel_applet_on_context_menu_item_clicked_bonobo_ui_verb_fn (BonoboUIComponent* component, void* user_data, const char* cname);
static gboolean _tuntun_panel_applet_on_button_press_release_gtk_widget_button_press_event (TuntunPanelApplet* _sender, GdkEventButton* event, gpointer self);
static gboolean _tuntun_panel_applet_on_query_tooltip_gtk_widget_query_tooltip (TuntunPanelApplet* _sender, gint x, gint y, gboolean keyboard_tooltip, GtkTooltip* tooltip, gpointer self);
static void tuntun_panel_applet_create (TuntunPanelApplet* self);
static gboolean tuntun_panel_applet_factory (TuntunPanelApplet* applet, const char* iid, void* data);
static gboolean tuntun_panel_applet_on_query_tooltip (TuntunPanelApplet* self, TuntunPanelApplet* applet, gint x, gint y, gboolean keyboard_tooltip, GtkTooltip* tooltip);
static gboolean tuntun_panel_applet_on_animation_timeout (TuntunPanelApplet* self);
static gboolean _tuntun_panel_applet_on_animation_timeout_gsource_func (gpointer self);
static void tuntun_panel_applet_animate_icon (TuntunPanelApplet* self);
static gboolean tuntun_panel_applet_on_button_press_release (TuntunPanelApplet* self, TuntunPanelApplet* sender, GdkEventButton* event);
static void tuntun_panel_applet_quick_connect (TuntunPanelApplet* self);
static void tuntun_panel_applet_on_connection_activity (TuntunPanelApplet* self, TuntunConnections* connections, TuntunConnection* connection);
static void tuntun_panel_applet_authenticate (TuntunPanelApplet* self, TuntunConnection* connection, TuntunAuthenticationModes mode, const char* type);
static void tuntun_panel_applet_on_connection_authentication_required (TuntunPanelApplet* self, TuntunConnections* connections, TuntunConnection* connection, TuntunAuthenticationModes mode, const char* type);
static void tuntun_panel_applet_on_connection_authentication_failed (TuntunPanelApplet* self, TuntunConnections* connections, TuntunConnection* connection, TuntunAuthenticationModes mode, const char* type);
static void tuntun_panel_applet_on_connection_fatal_error (TuntunPanelApplet* self, TuntunConnections* connections, TuntunConnection* connection, const char* error);
static void tuntun_panel_applet_on_connection_status_changed (TuntunPanelApplet* self, TuntunConnections* connections, TuntunConnection* connection);
static void _tuntun_panel_applet_on_connection_menu_item_activated_gtk_menu_item_activate (GtkImageMenuItem* _sender, gpointer self);
static void tuntun_panel_applet_select_connection (TuntunPanelApplet* self);
static void tuntun_panel_applet_on_connection_menu_item_activated (TuntunPanelApplet* self, GtkImageMenuItem* sender);
static void tuntun_panel_applet_on_context_menu_item_clicked (BonoboUIComponent* component, void* user_data, const char* cname);
static void _tuntun_panel_applet_on_log_window_closed_tuntun_log_window_closed (TuntunLogWindow* _sender, gpointer self);
static void tuntun_panel_applet_show_log (TuntunPanelApplet* self);
static void _tuntun_panel_applet_on_settings_dialog_closed_tuntun_connections_dialog_closed (TuntunConnectionsDialog* _sender, gpointer self);
static void tuntun_panel_applet_show_properties (TuntunPanelApplet* self);
static void tuntun_panel_applet_on_log_window_closed (TuntunPanelApplet* self, TuntunLogWindow* sender);
static void tuntun_panel_applet_on_settings_dialog_closed (TuntunPanelApplet* self, TuntunConnectionsDialog* sender);
static void tuntun_panel_applet_about (TuntunPanelApplet* self);
static gpointer tuntun_panel_applet_parent_class = NULL;
static void tuntun_panel_applet_dispose (GObject * obj);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static int _vala_strcmp0 (const char * str1, const char * str2);



TuntunPanelApplet* tuntun_panel_applet_new (void) {
	TuntunPanelApplet * self;
	self = g_object_newv (TUNTUN_TYPE_PANEL_APPLET, 0, NULL);
	return self;
}


static void _tuntun_panel_applet_on_connection_status_changed_tuntun_connections_connection_status_changed (TuntunConnections* _sender, TuntunConnection* connection, gpointer self) {
	tuntun_panel_applet_on_connection_status_changed (self, _sender, connection);
}


static void _tuntun_panel_applet_on_connection_fatal_error_tuntun_connections_connection_fatal_error (TuntunConnections* _sender, TuntunConnection* connection, const char* error, gpointer self) {
	tuntun_panel_applet_on_connection_fatal_error (self, _sender, connection, error);
}


static void _tuntun_panel_applet_on_connection_authentication_required_tuntun_connections_authentication_required (TuntunConnections* _sender, TuntunConnection* connection, TuntunAuthenticationModes mode, const char* type, gpointer self) {
	tuntun_panel_applet_on_connection_authentication_required (self, _sender, connection, mode, type);
}


static void _tuntun_panel_applet_on_connection_authentication_failed_tuntun_connections_authentication_failed (TuntunConnections* _sender, TuntunConnection* connection, TuntunAuthenticationModes mode, const char* type, gpointer self) {
	tuntun_panel_applet_on_connection_authentication_failed (self, _sender, connection, mode, type);
}


static void _tuntun_panel_applet_on_connection_activity_tuntun_connections_connection_activity (TuntunConnections* _sender, TuntunConnection* connection, gpointer self) {
	tuntun_panel_applet_on_connection_activity (self, _sender, connection);
}


static void _tuntun_panel_applet_on_context_menu_item_clicked_bonobo_ui_verb_fn (BonoboUIComponent* component, void* user_data, const char* cname) {
	tuntun_panel_applet_on_context_menu_item_clicked (component, user_data, cname);
}


static gboolean _tuntun_panel_applet_on_button_press_release_gtk_widget_button_press_event (TuntunPanelApplet* _sender, GdkEventButton* event, gpointer self) {
	return tuntun_panel_applet_on_button_press_release (self, _sender, event);
}


static gboolean _tuntun_panel_applet_on_query_tooltip_gtk_widget_query_tooltip (TuntunPanelApplet* _sender, gint x, gint y, gboolean keyboard_tooltip, GtkTooltip* tooltip, gpointer self) {
	return tuntun_panel_applet_on_query_tooltip (self, _sender, x, y, keyboard_tooltip, tooltip);
}


static void tuntun_panel_applet_create (TuntunPanelApplet* self) {
	GError * inner_error;
	g_return_if_fail (TUNTUN_IS_PANEL_APPLET (self));
	inner_error = NULL;
	{
		TuntunTuntun* _tmp0;
		BonoboUIVerb* _tmp1;
		GdkPixbuf** _tmp2;
		char* file;
		GdkPixbuf* _tmp3;
		char* _tmp4;
		GdkPixbuf* _tmp5;
		char* _tmp6;
		GdkPixbuf* _tmp7;
		char* _tmp8;
		char* _tmp9;
		char* _tmp10;
		char* _tmp11;
		GtkImage* _tmp12;
		char* _tmp16;
		char* _tmp15;
		char* _tmp14;
		char* _tmp13;
		char* _tmp17;
		notify_init ("Tuntun");
		_tmp0 = NULL;
		self->priv->_tuntun = (_tmp0 = tuntun_tuntun_new (), (self->priv->_tuntun == NULL ? NULL : (self->priv->_tuntun = (g_object_unref (self->priv->_tuntun), NULL))), _tmp0);
		g_signal_connect_object (tuntun_tuntun_get_connections (self->priv->_tuntun), "connection-status-changed", ((GCallback) (_tuntun_panel_applet_on_connection_status_changed_tuntun_connections_connection_status_changed)), self, 0);
		g_signal_connect_object (tuntun_tuntun_get_connections (self->priv->_tuntun), "connection-fatal-error", ((GCallback) (_tuntun_panel_applet_on_connection_fatal_error_tuntun_connections_connection_fatal_error)), self, 0);
		g_signal_connect_object (tuntun_tuntun_get_connections (self->priv->_tuntun), "authentication-required", ((GCallback) (_tuntun_panel_applet_on_connection_authentication_required_tuntun_connections_authentication_required)), self, 0);
		g_signal_connect_object (tuntun_tuntun_get_connections (self->priv->_tuntun), "authentication-failed", ((GCallback) (_tuntun_panel_applet_on_connection_authentication_failed_tuntun_connections_authentication_failed)), self, 0);
		g_signal_connect_object (tuntun_tuntun_get_connections (self->priv->_tuntun), "connection-activity", ((GCallback) (_tuntun_panel_applet_on_connection_activity_tuntun_connections_connection_activity)), self, 0);
		_tmp1 = NULL;
		self->priv->_verbs = (_tmp1 = g_new0 (BonoboUIVerb, 4), (self->priv->_verbs = (g_free (self->priv->_verbs), NULL)), self->priv->_verbs_length1 = 4, _tmp1);
		_tmp2 = NULL;
		self->priv->_images = (_tmp2 = g_new0 (GdkPixbuf*, 3 + 1), (self->priv->_images = (_vala_array_free (self->priv->_images, self->priv->_images_length1, ((GDestroyNotify) (g_object_unref))), NULL)), self->priv->_images_length1 = 3, _tmp2);
		file = tuntun_utils_get_image_path (TUNTUN_CONSTANTS_IMAGES_PANEL_ICON_ACTIVITY_1);
		_tmp3 = NULL;
		self->priv->_images[0] = (_tmp3 = gdk_pixbuf_new_from_file (file, &inner_error), (self->priv->_images[0] == NULL ? NULL : (self->priv->_images[0] = (g_object_unref (self->priv->_images[0]), NULL))), _tmp3);
		if (inner_error != NULL) {
			goto __catch0_g_error;
		}
		_tmp4 = NULL;
		file = (_tmp4 = tuntun_utils_get_image_path (TUNTUN_CONSTANTS_IMAGES_PANEL_ICON_NORMAL), (file = (g_free (file), NULL)), _tmp4);
		_tmp5 = NULL;
		self->priv->_images[1] = (_tmp5 = gdk_pixbuf_new_from_file (file, &inner_error), (self->priv->_images[1] == NULL ? NULL : (self->priv->_images[1] = (g_object_unref (self->priv->_images[1]), NULL))), _tmp5);
		if (inner_error != NULL) {
			goto __catch0_g_error;
		}
		_tmp6 = NULL;
		file = (_tmp6 = tuntun_utils_get_image_path (TUNTUN_CONSTANTS_IMAGES_PANEL_ICON_ACTIVITY_2), (file = (g_free (file), NULL)), _tmp6);
		_tmp7 = NULL;
		self->priv->_images[2] = (_tmp7 = gdk_pixbuf_new_from_file (file, &inner_error), (self->priv->_images[2] == NULL ? NULL : (self->priv->_images[2] = (g_object_unref (self->priv->_images[2]), NULL))), _tmp7);
		if (inner_error != NULL) {
			goto __catch0_g_error;
		}
		_tmp8 = NULL;
		self->priv->_verbs[0].cname = (_tmp8 = g_strdup ("Properties"), (self->priv->_verbs[0].cname = (g_free (self->priv->_verbs[0].cname), NULL)), _tmp8);
		self->priv->_verbs[0].cb = _tuntun_panel_applet_on_context_menu_item_clicked_bonobo_ui_verb_fn;
		self->priv->_verbs[0].user_data = self;
		_tmp9 = NULL;
		self->priv->_verbs[1].cname = (_tmp9 = g_strdup ("Log"), (self->priv->_verbs[1].cname = (g_free (self->priv->_verbs[1].cname), NULL)), _tmp9);
		self->priv->_verbs[1].cb = _tuntun_panel_applet_on_context_menu_item_clicked_bonobo_ui_verb_fn;
		self->priv->_verbs[1].user_data = self;
		_tmp10 = NULL;
		self->priv->_verbs[2].cname = (_tmp10 = g_strdup ("About"), (self->priv->_verbs[2].cname = (g_free (self->priv->_verbs[2].cname), NULL)), _tmp10);
		self->priv->_verbs[2].cb = _tuntun_panel_applet_on_context_menu_item_clicked_bonobo_ui_verb_fn;
		self->priv->_verbs[2].user_data = self;
		_tmp11 = NULL;
		self->priv->_verbs[3].cname = (_tmp11 = NULL, (self->priv->_verbs[3].cname = (g_free (self->priv->_verbs[3].cname), NULL)), _tmp11);
		self->priv->_verbs[3].cb = NULL;
		self->priv->_verbs[3].user_data = NULL;
		_tmp12 = NULL;
		self->priv->_image = (_tmp12 = g_object_ref_sink (((GtkImage*) (gtk_image_new_from_pixbuf (self->priv->_images[1])))), (self->priv->_image == NULL ? NULL : (self->priv->_image = (g_object_unref (self->priv->_image), NULL))), _tmp12);
		gtk_container_add (GTK_CONTAINER (self), GTK_WIDGET (self->priv->_image));
		_tmp16 = NULL;
		_tmp15 = NULL;
		_tmp14 = NULL;
		_tmp13 = NULL;
		self->priv->_right_click_menu_xml = (_tmp16 = g_strconcat ((_tmp15 = g_strconcat ((_tmp14 = g_strconcat ((_tmp13 = g_strconcat ("<popup name=\"button3\">", "<menuitem name=\"Properties Item\" verb=\"Properties\" _label=\"%s\" pixtype=\"stock\" pixname=\"gtk-properties\"/>", NULL)), "<menuitem name=\"Log Window\" verb=\"Log\" _label=\"%s\" pixtype=\"stock\" pixname=\"gtk-info\"/>", NULL)), "<menuitem name=\"About Item\" verb=\"About\" _label=\"%s\" pixtype=\"stock\" pixname=\"gnome-stock-about\"/>", NULL)), "</popup>", NULL), (self->priv->_right_click_menu_xml = (g_free (self->priv->_right_click_menu_xml), NULL)), _tmp16);
		_tmp15 = (g_free (_tmp15), NULL);
		_tmp14 = (g_free (_tmp14), NULL);
		_tmp13 = (g_free (_tmp13), NULL);
		_tmp17 = NULL;
		panel_applet_setup_menu (PANEL_APPLET (self), (_tmp17 = g_strdup_printf (self->priv->_right_click_menu_xml, _ ("_Preferences..."), _ ("_Show log..."), _ ("_About..."))), self->priv->_verbs, self);
		_tmp17 = (g_free (_tmp17), NULL);
		g_signal_connect_object (GTK_WIDGET (self), "button-press-event", ((GCallback) (_tuntun_panel_applet_on_button_press_release_gtk_widget_button_press_event)), self, 0);
		gtk_widget_set_has_tooltip (GTK_WIDGET (self), TRUE);
		g_signal_connect_object (GTK_WIDGET (self), "query-tooltip", ((GCallback) (_tuntun_panel_applet_on_query_tooltip_gtk_widget_query_tooltip)), self, 0);
		gtk_widget_show_all (GTK_WIDGET (self));
		file = (g_free (file), NULL);
	}
	goto __finally0;
	__catch0_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			tuntun_utils_display_error ("PanelApplet.create", err->message);
			(err == NULL ? NULL : (err = (g_error_free (err), NULL)));
		}
	}
	__finally0:
	;
}


static gboolean tuntun_panel_applet_factory (TuntunPanelApplet* applet, const char* iid, void* data) {
	g_return_val_if_fail (TUNTUN_IS_PANEL_APPLET (applet), FALSE);
	g_return_val_if_fail (iid != NULL, FALSE);
	tuntun_panel_applet_create (applet);
	return TRUE;
}


static gboolean tuntun_panel_applet_on_query_tooltip (TuntunPanelApplet* self, TuntunPanelApplet* applet, gint x, gint y, gboolean keyboard_tooltip, GtkTooltip* tooltip) {
	TuntunTooltip* _tmp0;
	g_return_val_if_fail (TUNTUN_IS_PANEL_APPLET (self), FALSE);
	g_return_val_if_fail (TUNTUN_IS_PANEL_APPLET (applet), FALSE);
	g_return_val_if_fail (GTK_IS_TOOLTIP (tooltip), FALSE);
	_tmp0 = NULL;
	gtk_tooltip_set_custom (tooltip, GTK_WIDGET ((_tmp0 = g_object_ref_sink (tuntun_tooltip_new (tuntun_tuntun_get_connections (self->priv->_tuntun))))));
	(_tmp0 == NULL ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL)));
	return TRUE;
}


static gboolean tuntun_panel_applet_on_animation_timeout (TuntunPanelApplet* self) {
	g_return_val_if_fail (TUNTUN_IS_PANEL_APPLET (self), FALSE);
	if (self->priv->_animation_status == 1) {
		self->priv->_animation_status = 0;
		return FALSE;
	} else {
		self->priv->_image_idx++;
		if (self->priv->_image_idx >= self->priv->_images_length1) {
			self->priv->_image_idx = 0;
		} else {
			if (self->priv->_image_idx == 1) {
				self->priv->_animation_status--;
			}
		}
		gtk_image_set_from_pixbuf (self->priv->_image, self->priv->_images[self->priv->_image_idx]);
		return TRUE;
	}
}


static gboolean _tuntun_panel_applet_on_animation_timeout_gsource_func (gpointer self) {
	return tuntun_panel_applet_on_animation_timeout (self);
}


static void tuntun_panel_applet_animate_icon (TuntunPanelApplet* self) {
	g_return_if_fail (TUNTUN_IS_PANEL_APPLET (self));
	if (self->priv->_animation_status == 0) {
		g_timeout_add (((guint) (250)), _tuntun_panel_applet_on_animation_timeout_gsource_func, self);
		self->priv->_image_idx = 1;
	}
	self->priv->_animation_status = 4;
}


static gboolean tuntun_panel_applet_on_button_press_release (TuntunPanelApplet* self, TuntunPanelApplet* sender, GdkEventButton* event) {
	g_return_val_if_fail (TUNTUN_IS_PANEL_APPLET (self), FALSE);
	g_return_val_if_fail (TUNTUN_IS_PANEL_APPLET (sender), FALSE);
	if ((*event).type == GDK_BUTTON_PRESS && (*event).button == 1) {
		if (((*event).state & GDK_SHIFT_MASK) != 0) {
			tuntun_panel_applet_quick_connect (self);
		} else {
			tuntun_panel_applet_select_connection (self);
		}
		return TRUE;
	}
	return FALSE;
}


static void tuntun_panel_applet_quick_connect (TuntunPanelApplet* self) {
	g_return_if_fail (TUNTUN_IS_PANEL_APPLET (self));
	{
		GList* conn_collection;
		GList* conn_it;
		conn_collection = tuntun_tuntun_get_connections (self->priv->_tuntun)->items;
		for (conn_it = conn_collection; conn_it != NULL; conn_it = conn_it->next) {
			TuntunConnection* _tmp0;
			TuntunConnection* conn;
			_tmp0 = NULL;
			conn = (_tmp0 = ((TuntunConnection*) (conn_it->data)), (_tmp0 == NULL ? NULL : g_object_ref (_tmp0)));
			{
				if (tuntun_connection_info_get_quick_connect (tuntun_connection_get_info (conn)) == TRUE) {
					if (tuntun_connection_get_status (conn) == TUNTUN_CONNECTION_STATES_DISCONNECTED) {
						tuntun_connection_connect (conn);
					} else {
						tuntun_connection_disconnect (conn);
					}
				}
				(conn == NULL ? NULL : (conn = (g_object_unref (conn), NULL)));
			}
		}
	}
}


static void tuntun_panel_applet_on_connection_activity (TuntunPanelApplet* self, TuntunConnections* connections, TuntunConnection* connection) {
	g_return_if_fail (TUNTUN_IS_PANEL_APPLET (self));
	g_return_if_fail (TUNTUN_IS_CONNECTIONS (connections));
	g_return_if_fail (TUNTUN_IS_CONNECTION (connection));
	tuntun_panel_applet_animate_icon (self);
}


static void tuntun_panel_applet_authenticate (TuntunPanelApplet* self, TuntunConnection* connection, TuntunAuthenticationModes mode, const char* type) {
	TuntunAuthDialog* auth;
	g_return_if_fail (TUNTUN_IS_PANEL_APPLET (self));
	g_return_if_fail (TUNTUN_IS_CONNECTION (connection));
	g_return_if_fail (type != NULL);
	auth = g_object_ref_sink (tuntun_auth_dialog_new (connection, mode, type));
	tuntun_auth_dialog_authenticate (auth);
	(auth == NULL ? NULL : (auth = (g_object_unref (auth), NULL)));
}


static void tuntun_panel_applet_on_connection_authentication_required (TuntunPanelApplet* self, TuntunConnections* connections, TuntunConnection* connection, TuntunAuthenticationModes mode, const char* type) {
	g_return_if_fail (TUNTUN_IS_PANEL_APPLET (self));
	g_return_if_fail (TUNTUN_IS_CONNECTIONS (connections));
	g_return_if_fail (TUNTUN_IS_CONNECTION (connection));
	g_return_if_fail (type != NULL);
	tuntun_panel_applet_authenticate (self, connection, mode, type);
}


static void tuntun_panel_applet_on_connection_authentication_failed (TuntunPanelApplet* self, TuntunConnections* connections, TuntunConnection* connection, TuntunAuthenticationModes mode, const char* type) {
	GtkMessageDialog* dialog;
	char* _tmp0;
	g_return_if_fail (TUNTUN_IS_PANEL_APPLET (self));
	g_return_if_fail (TUNTUN_IS_CONNECTIONS (connections));
	g_return_if_fail (TUNTUN_IS_CONNECTION (connection));
	g_return_if_fail (type != NULL);
	/*suppress dialog if keyring authentication*/
	dialog = g_object_ref_sink (((GtkMessageDialog*) (gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, tuntun_connection_info_get_name (tuntun_connection_get_info (connection)), NULL))));
	_tmp0 = NULL;
	g_object_set (dialog, "secondary-text", _ ("Authentication failed"), NULL);
	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_object_destroy (GTK_OBJECT (dialog));
	(dialog == NULL ? NULL : (dialog = (g_object_unref (dialog), NULL)));
}


static void tuntun_panel_applet_on_connection_fatal_error (TuntunPanelApplet* self, TuntunConnections* connections, TuntunConnection* connection, const char* error) {
	GtkMessageDialog* dialog;
	char* _tmp0;
	g_return_if_fail (TUNTUN_IS_PANEL_APPLET (self));
	g_return_if_fail (TUNTUN_IS_CONNECTIONS (connections));
	g_return_if_fail (TUNTUN_IS_CONNECTION (connection));
	g_return_if_fail (error != NULL);
	dialog = g_object_ref_sink (((GtkMessageDialog*) (gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, tuntun_connection_info_get_name (tuntun_connection_get_info (connection)), NULL))));
	_tmp0 = NULL;
	g_object_set (dialog, "secondary-text", error, NULL);
	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_object_destroy (GTK_OBJECT (dialog));
	(dialog == NULL ? NULL : (dialog = (g_object_unref (dialog), NULL)));
}


static void tuntun_panel_applet_on_connection_status_changed (TuntunPanelApplet* self, TuntunConnections* connections, TuntunConnection* connection) {
	GError * inner_error;
	NotifyNotification* notification;
	g_return_if_fail (TUNTUN_IS_PANEL_APPLET (self));
	g_return_if_fail (TUNTUN_IS_CONNECTIONS (connections));
	g_return_if_fail (TUNTUN_IS_CONNECTION (connection));
	inner_error = NULL;
	notification = NULL;
	{
		if (tuntun_connection_get_status (connection) == TUNTUN_CONNECTION_STATES_CONNECTED) {
			NotifyNotification* _tmp2;
			char* _tmp1;
			char* _tmp0;
			_tmp2 = NULL;
			_tmp1 = NULL;
			_tmp0 = NULL;
			notification = (_tmp2 = notify_notification_new (tuntun_connection_info_get_name (tuntun_connection_get_info (connection)), (_tmp0 = g_strdup_printf (_ ("connection established\nassigned ip: %s"), tuntun_connection_info_get_assigned_ip (tuntun_connection_get_info (connection)))), (_tmp1 = tuntun_utils_get_image_path (TUNTUN_CONSTANTS_IMAGES_CONNECTION_STATUS_CONNECTED)), GTK_WIDGET (self)), (notification == NULL ? NULL : (notification = (g_object_unref (notification), NULL))), _tmp2);
			_tmp1 = (g_free (_tmp1), NULL);
			_tmp0 = (g_free (_tmp0), NULL);
		} else {
			if (tuntun_connection_get_status (connection) == TUNTUN_CONNECTION_STATES_DISCONNECTED) {
				NotifyNotification* _tmp4;
				char* _tmp3;
				_tmp4 = NULL;
				_tmp3 = NULL;
				notification = (_tmp4 = notify_notification_new (tuntun_connection_info_get_name (tuntun_connection_get_info (connection)), _ ("connection closed"), (_tmp3 = tuntun_utils_get_image_path (TUNTUN_CONSTANTS_IMAGES_CONNECTION_STATUS_DISCONNECTED)), GTK_WIDGET (self)), (notification == NULL ? NULL : (notification = (g_object_unref (notification), NULL))), _tmp4);
				_tmp3 = (g_free (_tmp3), NULL);
			}
		}
		if (notification != NULL) {
			notify_notification_set_urgency (notification, NOTIFY_URGENCY_NORMAL);
			notify_notification_set_timeout (notification, NOTIFY_EXPIRES_DEFAULT);
			notify_notification_show (notification, &inner_error);
			if (inner_error != NULL) {
				goto __catch1_g_error;
			}
		}
	}
	goto __finally1;
	__catch1_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_warning ("tuntun-applet.vala:240: error %d: %s", err->code, err->message);
			(err == NULL ? NULL : (err = (g_error_free (err), NULL)));
		}
	}
	__finally1:
	;
	(notification == NULL ? NULL : (notification = (g_object_unref (notification), NULL)));
}


static void _tuntun_panel_applet_on_connection_menu_item_activated_gtk_menu_item_activate (GtkImageMenuItem* _sender, gpointer self) {
	tuntun_panel_applet_on_connection_menu_item_activated (self, _sender);
}


static void tuntun_panel_applet_select_connection (TuntunPanelApplet* self) {
	GtkMenu* popup_menu;
	g_return_if_fail (TUNTUN_IS_PANEL_APPLET (self));
	popup_menu = g_object_ref_sink (((GtkMenu*) (gtk_menu_new ())));
	{
		GList* connection_collection;
		GList* connection_it;
		connection_collection = tuntun_tuntun_get_connections (self->priv->_tuntun)->items;
		for (connection_it = connection_collection; connection_it != NULL; connection_it = connection_it->next) {
			TuntunConnection* connection;
			connection = ((TuntunConnection*) (connection_it->data));
			{
				GtkImageMenuItem* item;
				gboolean _tmp0;
				char* menu_image;
				TuntunConnectionStates _tmp7;
				GtkImage* _tmp8;
				item = g_object_ref_sink (((GtkImageMenuItem*) (gtk_image_menu_item_new_with_label (tuntun_connection_info_get_name (tuntun_connection_get_info (connection))))));
				g_object_set (G_OBJECT (item), "user-data", connection, NULL);
				g_object_set (GTK_WIDGET (item), "sensitive", (tuntun_connection_get_control_channel_status (connection) == TUNTUN_CONNECTION_STATES_CONNECTED && tuntun_connection_get_status (connection) != TUNTUN_CONNECTION_STATES_CONNECTING && tuntun_connection_get_status (connection) != TUNTUN_CONNECTION_STATES_DISCONNECTING), NULL);
				gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), ((GtkWidget*) (GTK_MENU_ITEM (item))));
				menu_image = NULL;
				_tmp7 = tuntun_connection_get_status (connection);
				if (_tmp7 == TUNTUN_CONNECTION_STATES_UNKNOWN)
				do {
					char* _tmp1;
					_tmp1 = NULL;
					menu_image = (_tmp1 = tuntun_utils_get_image_path (TUNTUN_CONSTANTS_IMAGES_CONNECTION_STATUS_UNKNOWN), (menu_image = (g_free (menu_image), NULL)), _tmp1);
					break;
				} while (0); else if (_tmp7 == TUNTUN_CONNECTION_STATES_ERROR)
				do {
					char* _tmp2;
					_tmp2 = NULL;
					menu_image = (_tmp2 = tuntun_utils_get_image_path (TUNTUN_CONSTANTS_IMAGES_CONNECTION_STATUS_UNKNOWN), (menu_image = (g_free (menu_image), NULL)), _tmp2);
					break;
				} while (0); else if (_tmp7 == TUNTUN_CONNECTION_STATES_DISCONNECTED)
				do {
					char* _tmp3;
					_tmp3 = NULL;
					menu_image = (_tmp3 = tuntun_utils_get_image_path (TUNTUN_CONSTANTS_IMAGES_CONNECTION_STATUS_DISCONNECT), (menu_image = (g_free (menu_image), NULL)), _tmp3);
					break;
				} while (0); else if (_tmp7 == TUNTUN_CONNECTION_STATES_CONNECTED)
				do {
					char* _tmp4;
					_tmp4 = NULL;
					menu_image = (_tmp4 = tuntun_utils_get_image_path (TUNTUN_CONSTANTS_IMAGES_CONNECTION_STATUS_CONNECT), (menu_image = (g_free (menu_image), NULL)), _tmp4);
					break;
				} while (0); else if (_tmp7 == TUNTUN_CONNECTION_STATES_CONNECTING || _tmp7 == TUNTUN_CONNECTION_STATES_DISCONNECTING)
				do {
					char* _tmp5;
					_tmp5 = NULL;
					menu_image = (_tmp5 = tuntun_utils_get_image_path (TUNTUN_CONSTANTS_IMAGES_CONNECTION_STATUS_UNKNOWN), (menu_image = (g_free (menu_image), NULL)), _tmp5);
					break;
				} while (0); else
				do {
					char* _tmp6;
					_tmp6 = NULL;
					menu_image = (_tmp6 = tuntun_utils_get_image_path (TUNTUN_CONSTANTS_IMAGES_CONNECTION_STATUS_UNKNOWN), (menu_image = (g_free (menu_image), NULL)), _tmp6);
					break;
				} while (0);
				_tmp8 = NULL;
				gtk_image_menu_item_set_image (item, GTK_WIDGET ((_tmp8 = g_object_ref_sink (((GtkImage*) (gtk_image_new_from_file (menu_image)))))));
				(_tmp8 == NULL ? NULL : (_tmp8 = (g_object_unref (_tmp8), NULL)));
				g_signal_connect_object (GTK_MENU_ITEM (item), "activate", ((GCallback) (_tuntun_panel_applet_on_connection_menu_item_activated_gtk_menu_item_activate)), self, 0);
				(item == NULL ? NULL : (item = (g_object_unref (item), NULL)));
				menu_image = (g_free (menu_image), NULL);
			}
		}
	}
	gtk_widget_show_all (GTK_WIDGET (popup_menu));
	gtk_menu_popup (popup_menu, NULL, NULL, NULL, NULL, ((guint) (0)), gtk_get_current_event_time ());
	(popup_menu == NULL ? NULL : (popup_menu = (g_object_unref (popup_menu), NULL)));
}


static void tuntun_panel_applet_on_connection_menu_item_activated (TuntunPanelApplet* self, GtkImageMenuItem* sender) {
	TuntunConnection* connection;
	g_return_if_fail (TUNTUN_IS_PANEL_APPLET (self));
	g_return_if_fail (GTK_IS_IMAGE_MENU_ITEM (sender));
	connection = NULL;
	g_object_get (G_OBJECT (sender), "user-data", &connection, NULL);
	g_return_if_fail (connection != NULL);
	if (tuntun_connection_get_status (connection) == TUNTUN_CONNECTION_STATES_CONNECTED) {
		tuntun_connection_disconnect (connection);
	} else {
		if (tuntun_connection_get_status (connection) == TUNTUN_CONNECTION_STATES_DISCONNECTED) {
			tuntun_connection_connect (connection);
		}
	}
}


static void tuntun_panel_applet_on_context_menu_item_clicked (BonoboUIComponent* component, void* user_data, const char* cname) {
	TuntunPanelApplet* _tmp0;
	TuntunPanelApplet* instance;
	g_return_if_fail (component != NULL);
	g_return_if_fail (cname != NULL);
	_tmp0 = NULL;
	instance = (_tmp0 = TUNTUN_PANEL_APPLET (user_data), (_tmp0 == NULL ? NULL : g_object_ref (_tmp0)));
	if (_vala_strcmp0 (cname, "About") == 0) {
		tuntun_panel_applet_about (instance);
	} else {
		if (_vala_strcmp0 (cname, "Properties") == 0) {
			tuntun_panel_applet_show_properties (instance);
		} else {
			if (_vala_strcmp0 (cname, "Log") == 0) {
				tuntun_panel_applet_show_log (instance);
			}
		}
	}
	(instance == NULL ? NULL : (instance = (g_object_unref (instance), NULL)));
}


static void _tuntun_panel_applet_on_log_window_closed_tuntun_log_window_closed (TuntunLogWindow* _sender, gpointer self) {
	tuntun_panel_applet_on_log_window_closed (self, _sender);
}


static void tuntun_panel_applet_show_log (TuntunPanelApplet* self) {
	g_return_if_fail (TUNTUN_IS_PANEL_APPLET (self));
	if (self->priv->_log == NULL) {
		TuntunLogWindow* _tmp0;
		_tmp0 = NULL;
		self->priv->_log = (_tmp0 = tuntun_log_window_new (self->priv->_tuntun), (self->priv->_log == NULL ? NULL : (self->priv->_log = (g_object_unref (self->priv->_log), NULL))), _tmp0);
		g_signal_connect_object (self->priv->_log, "closed", ((GCallback) (_tuntun_panel_applet_on_log_window_closed_tuntun_log_window_closed)), self, 0);
	}
	tuntun_log_window_show (self->priv->_log);
}


static void _tuntun_panel_applet_on_settings_dialog_closed_tuntun_connections_dialog_closed (TuntunConnectionsDialog* _sender, gpointer self) {
	tuntun_panel_applet_on_settings_dialog_closed (self, _sender);
}


static void tuntun_panel_applet_show_properties (TuntunPanelApplet* self) {
	g_return_if_fail (TUNTUN_IS_PANEL_APPLET (self));
	if (self->priv->_settings == NULL) {
		TuntunConnectionsDialog* _tmp0;
		_tmp0 = NULL;
		self->priv->_settings = (_tmp0 = tuntun_connections_dialog_new (self->priv->_tuntun), (self->priv->_settings == NULL ? NULL : (self->priv->_settings = (g_object_unref (self->priv->_settings), NULL))), _tmp0);
		g_signal_connect_object (self->priv->_settings, "closed", ((GCallback) (_tuntun_panel_applet_on_settings_dialog_closed_tuntun_connections_dialog_closed)), self, 0);
	}
	tuntun_connections_dialog_show (self->priv->_settings);
}


static void tuntun_panel_applet_on_log_window_closed (TuntunPanelApplet* self, TuntunLogWindow* sender) {
	guint _tmp0;
	TuntunLogWindow* _tmp1;
	g_return_if_fail (TUNTUN_IS_PANEL_APPLET (self));
	g_return_if_fail (TUNTUN_IS_LOG_WINDOW (sender));
	g_signal_handlers_disconnect_matched (self->priv->_log, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("closed", TUNTUN_TYPE_LOG_WINDOW, &_tmp0, NULL, FALSE), _tmp0), 0, NULL, ((GCallback) (_tuntun_panel_applet_on_log_window_closed_tuntun_log_window_closed)), self);
	_tmp1 = NULL;
	self->priv->_log = (_tmp1 = NULL, (self->priv->_log == NULL ? NULL : (self->priv->_log = (g_object_unref (self->priv->_log), NULL))), _tmp1);
}


static void tuntun_panel_applet_on_settings_dialog_closed (TuntunPanelApplet* self, TuntunConnectionsDialog* sender) {
	guint _tmp0;
	TuntunConnectionsDialog* _tmp1;
	g_return_if_fail (TUNTUN_IS_PANEL_APPLET (self));
	g_return_if_fail (TUNTUN_IS_CONNECTIONS_DIALOG (sender));
	g_signal_handlers_disconnect_matched (self->priv->_settings, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("closed", TUNTUN_TYPE_CONNECTIONS_DIALOG, &_tmp0, NULL, FALSE), _tmp0), 0, NULL, ((GCallback) (_tuntun_panel_applet_on_settings_dialog_closed_tuntun_connections_dialog_closed)), self);
	_tmp1 = NULL;
	self->priv->_settings = (_tmp1 = NULL, (self->priv->_settings == NULL ? NULL : (self->priv->_settings = (g_object_unref (self->priv->_settings), NULL))), _tmp1);
}


static void tuntun_panel_applet_about (TuntunPanelApplet* self) {
	GtkAboutDialog* dialog;
	char** _tmp1;
	gint authors_length1;
	char** _tmp0;
	char** authors;
	const char* _tmp2;
	char* translator_credits;
	g_return_if_fail (TUNTUN_IS_PANEL_APPLET (self));
	dialog = g_object_ref_sink (((GtkAboutDialog*) (gtk_about_dialog_new ())));
	_tmp1 = NULL;
	_tmp0 = NULL;
	authors = (_tmp1 = (_tmp0 = g_new0 (char*, 2 + 1), _tmp0[0] = g_strdup ("Andrea Del Signore"), _tmp0[1] = NULL, _tmp0), authors_length1 = 2, _tmp1);
	_tmp2 = NULL;
	translator_credits = (_tmp2 = _ ("translator_credits"), (_tmp2 == NULL ? NULL : g_strdup (_tmp2)));
	if (g_str_has_prefix ("translator_credits", translator_credits) == FALSE) {
		gtk_about_dialog_set_translator_credits (dialog, translator_credits);
	}
	gtk_buildable_set_name (GTK_BUILDABLE (dialog), PACKAGE_STRING);
	gtk_about_dialog_set_comments (dialog, _ ("TunTun VPN Connection manager"));
	gtk_about_dialog_set_version (dialog, PACKAGE_VERSION);
	gtk_about_dialog_set_website (dialog, "http://code.google.com/p/tuntun");
	gtk_about_dialog_set_website_label (dialog, _ ("Homepage"));
	gtk_about_dialog_set_license (dialog, _ ("(c) 2008 Andrea Del Signore (sejerpz@tin.it).\n\n This program is licensed under the terms of the GNU\n\n General Public License and is provided with absolutely\n\n NO WARRANTY; see the file COPYING for details."));
	gtk_about_dialog_set_authors (dialog, authors);
	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_hide (GTK_WIDGET (dialog));
	(dialog == NULL ? NULL : (dialog = (g_object_unref (dialog), NULL)));
	authors = (_vala_array_free (authors, authors_length1, ((GDestroyNotify) (g_free))), NULL);
	translator_credits = (g_free (translator_credits), NULL);
}


gint tuntun_panel_applet_main (char** args, int args_length1) {
	GnomeProgram* _tmp0;
	GnomeProgram* program;
	gint ret;
	gint _tmp1;
	_tmp0 = NULL;
	program = (_tmp0 = gnome_program_init (PACKAGE_STRING, "0", LIBGNOMEUI_MODULE, args_length1, args, "sm-connect", FALSE, NULL), (_tmp0 == NULL ? NULL : g_object_ref (_tmp0)));
	ret = panel_applet_factory_main ("OAFIID:GNOME_Tuntun_Factory", TUNTUN_TYPE_PANEL_APPLET, ((PanelAppletFactoryCallback) (tuntun_panel_applet_factory)), NULL);
	return (_tmp1 = ret, (program == NULL ? NULL : (program = (g_object_unref (program), NULL))), _tmp1);
}


int main (int argc, char ** argv) {
	g_type_init ();
	return tuntun_panel_applet_main (argv, argc);
}


static void tuntun_panel_applet_class_init (TuntunPanelAppletClass * klass) {
	tuntun_panel_applet_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (TuntunPanelAppletPrivate));
	G_OBJECT_CLASS (klass)->dispose = tuntun_panel_applet_dispose;
}


static void tuntun_panel_applet_instance_init (TuntunPanelApplet * self) {
	self->priv = TUNTUN_PANEL_APPLET_GET_PRIVATE (self);
	self->priv->_verbs = NULL;
	self->priv->_settings = NULL;
	self->priv->_log = NULL;
	self->priv->_image_idx = 1;
	self->priv->_animation_status = 0;
}


static void tuntun_panel_applet_dispose (GObject * obj) {
	TuntunPanelApplet * self;
	self = TUNTUN_PANEL_APPLET (obj);
	self->priv->_right_click_menu_xml = (g_free (self->priv->_right_click_menu_xml), NULL);
	self->priv->_verbs = (g_free (self->priv->_verbs), NULL);
	(self->priv->_tuntun == NULL ? NULL : (self->priv->_tuntun = (g_object_unref (self->priv->_tuntun), NULL)));
	(self->priv->_settings == NULL ? NULL : (self->priv->_settings = (g_object_unref (self->priv->_settings), NULL)));
	(self->priv->_log == NULL ? NULL : (self->priv->_log = (g_object_unref (self->priv->_log), NULL)));
	(self->priv->_image == NULL ? NULL : (self->priv->_image = (g_object_unref (self->priv->_image), NULL)));
	self->priv->_images = (_vala_array_free (self->priv->_images, self->priv->_images_length1, ((GDestroyNotify) (g_object_unref))), NULL);
	G_OBJECT_CLASS (tuntun_panel_applet_parent_class)->dispose (obj);
}


GType tuntun_panel_applet_get_type (void) {
	static volatile gsize tuntun_panel_applet_type_id = 0;
	if (g_once_init_enter (&tuntun_panel_applet_type_id)) {
		GType tuntun_panel_applet_type_id_temp;
		static const GTypeInfo g_define_type_info = { sizeof (TuntunPanelAppletClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) tuntun_panel_applet_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (TuntunPanelApplet), 0, (GInstanceInitFunc) tuntun_panel_applet_instance_init };
		tuntun_panel_applet_type_id_temp = g_type_register_static (PANEL_TYPE_APPLET, "TuntunPanelApplet", &g_define_type_info, 0);
		g_once_init_leave (&tuntun_panel_applet_type_id, tuntun_panel_applet_type_id_temp);
	}
	return tuntun_panel_applet_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if (array != NULL && destroy_func != NULL) {
		int i;
		if (array_length >= 0)
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) (array))[i] != NULL)
			destroy_func (((gpointer*) (array))[i]);
		}
		else
		for (i = 0; ((gpointer*) (array))[i] != NULL; i = i + 1) {
			destroy_func (((gpointer*) (array))[i]);
		}
	}
	g_free (array);
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return (str1 != str2);
	}
	return strcmp (str1, str2);
}




