/*
 *  tuntun-connection-dialog.vala is a part of Tuntun
 *
 *  Tuntun: a simple applet to manage OpenVPN connections
 *
 *  Copyright (C) 2008  Andrea Del Signore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
 *
 *  Author:
 *     Andrea Del Signore <sejerpz@tin.it>       
 */

#include "tuntun-connection-dialog.h"
#include <stdlib.h>
#include <string.h>
#include "tuntun-utils.h"




struct _TuntunConnectionDialogPrivate {
	GtkDialog* _window;
	GtkEntry* _name;
	GtkEntry* _address;
	GtkEntry* _port;
	GtkListStore* _store;
	TuntunConnectionInfo* _connection_info;
};

#define TUNTUN_CONNECTION_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TUNTUN_TYPE_CONNECTION_DIALOG, TuntunConnectionDialogPrivate))
enum  {
	TUNTUN_CONNECTION_DIALOG_DUMMY_PROPERTY,
	TUNTUN_CONNECTION_DIALOG_CONNECTION_INFO
};
static void _tuntun_connection_dialog_on_button_connection_ok_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self);
static void _tuntun_connection_dialog_on_button_connection_cancel_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self);
static void tuntun_connection_dialog_initialize (TuntunConnectionDialog* self);
static void tuntun_connection_dialog_cleanup (TuntunConnectionDialog* self);
static void tuntun_connection_dialog_on_button_connection_ok_clicked (TuntunConnectionDialog* self, GtkButton* sender);
static void tuntun_connection_dialog_on_button_connection_cancel_clicked (TuntunConnectionDialog* self, GtkButton* sender);
static void tuntun_connection_dialog_update_text_entries (TuntunConnectionDialog* self);
static void tuntun_connection_dialog_update_connection_info_object (TuntunConnectionDialog* self);
static void tuntun_connection_dialog_set_text_if_null (TuntunConnectionDialog* self, GtkEntry* entry, const char* text);
static void tuntun_connection_dialog_set_connection_info (TuntunConnectionDialog* self, TuntunConnectionInfo* value);
static GObject * tuntun_connection_dialog_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer tuntun_connection_dialog_parent_class = NULL;
static void tuntun_connection_dialog_dispose (GObject * obj);



static void _tuntun_connection_dialog_on_button_connection_ok_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self) {
	tuntun_connection_dialog_on_button_connection_ok_clicked (self, _sender);
}


static void _tuntun_connection_dialog_on_button_connection_cancel_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self) {
	tuntun_connection_dialog_on_button_connection_cancel_clicked (self, _sender);
}


static void tuntun_connection_dialog_initialize (TuntunConnectionDialog* self) {
	GtkBuilder* _tmp0;
	GtkBuilder* builder;
	GtkButton* _tmp1;
	GtkButton* button;
	GtkButton* _tmp3;
	GtkButton* _tmp2;
	g_return_if_fail (TUNTUN_IS_CONNECTION_DIALOG (self));
	_tmp0 = NULL;
	builder = (_tmp0 = tuntun_utils_get_ui (), (_tmp0 == NULL ? NULL : g_object_ref (_tmp0)));
	self->priv->_window = GTK_DIALOG (gtk_builder_get_object (builder, "dialog_connection_properties"));
	g_assert (self->priv->_window != NULL);
	_tmp1 = NULL;
	button = (_tmp1 = GTK_BUTTON (gtk_builder_get_object (builder, "button_connection_dialog_ok")), (_tmp1 == NULL ? NULL : g_object_ref (_tmp1)));
	g_assert (button != NULL);
	g_signal_connect_object (button, "clicked", ((GCallback) (_tuntun_connection_dialog_on_button_connection_ok_clicked_gtk_button_clicked)), self, 0);
	_tmp3 = NULL;
	_tmp2 = NULL;
	button = (_tmp3 = (_tmp2 = GTK_BUTTON (gtk_builder_get_object (builder, "button_connection_dialog_cancel")), (_tmp2 == NULL ? NULL : g_object_ref (_tmp2))), (button == NULL ? NULL : (button = (g_object_unref (button), NULL))), _tmp3);
	g_assert (button != NULL);
	g_signal_connect_object (button, "clicked", ((GCallback) (_tuntun_connection_dialog_on_button_connection_cancel_clicked_gtk_button_clicked)), self, 0);
	self->priv->_name = GTK_ENTRY (gtk_builder_get_object (builder, "entry_connection_name"));
	g_assert (self->priv->_name != NULL);
	self->priv->_address = GTK_ENTRY (gtk_builder_get_object (builder, "entry_connection_host"));
	g_assert (self->priv->_address != NULL);
	self->priv->_port = GTK_ENTRY (gtk_builder_get_object (builder, "entry_connection_port"));
	g_assert (self->priv->_port != NULL);
	tuntun_connection_dialog_update_text_entries (self);
	(builder == NULL ? NULL : (builder = (g_object_unref (builder), NULL)));
	(button == NULL ? NULL : (button = (g_object_unref (button), NULL)));
}


static void tuntun_connection_dialog_cleanup (TuntunConnectionDialog* self) {
	GtkBuilder* _tmp0;
	GtkBuilder* builder;
	GtkButton* _tmp1;
	GtkButton* button;
	guint _tmp2;
	GtkButton* _tmp4;
	GtkButton* _tmp3;
	guint _tmp5;
	g_return_if_fail (TUNTUN_IS_CONNECTION_DIALOG (self));
	_tmp0 = NULL;
	builder = (_tmp0 = tuntun_utils_get_ui (), (_tmp0 == NULL ? NULL : g_object_ref (_tmp0)));
	_tmp1 = NULL;
	button = (_tmp1 = GTK_BUTTON (gtk_builder_get_object (builder, "button_connection_dialog_ok")), (_tmp1 == NULL ? NULL : g_object_ref (_tmp1)));
	g_signal_handlers_disconnect_matched (button, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("clicked", GTK_TYPE_BUTTON, &_tmp2, NULL, FALSE), _tmp2), 0, NULL, ((GCallback) (_tuntun_connection_dialog_on_button_connection_ok_clicked_gtk_button_clicked)), self);
	_tmp4 = NULL;
	_tmp3 = NULL;
	button = (_tmp4 = (_tmp3 = GTK_BUTTON (gtk_builder_get_object (builder, "button_connection_dialog_cancel")), (_tmp3 == NULL ? NULL : g_object_ref (_tmp3))), (button == NULL ? NULL : (button = (g_object_unref (button), NULL))), _tmp4);
	g_signal_handlers_disconnect_matched (button, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("clicked", GTK_TYPE_BUTTON, &_tmp5, NULL, FALSE), _tmp5), 0, NULL, ((GCallback) (_tuntun_connection_dialog_on_button_connection_cancel_clicked_gtk_button_clicked)), self);
	(builder == NULL ? NULL : (builder = (g_object_unref (builder), NULL)));
	(button == NULL ? NULL : (button = (g_object_unref (button), NULL)));
}


TuntunConnectionDialog* tuntun_connection_dialog_new (TuntunConnectionInfo* connection_info) {
	GParameter * __params;
	GParameter * __params_it;
	TuntunConnectionDialog * self;
	g_return_val_if_fail (TUNTUN_IS_CONNECTION_INFO (connection_info), NULL);
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
	__params_it->name = "connection-info";
	g_value_init (&__params_it->value, TUNTUN_TYPE_CONNECTION_INFO);
	g_value_set_object (&__params_it->value, connection_info);
	__params_it++;
	self = g_object_newv (TUNTUN_TYPE_CONNECTION_DIALOG, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


GtkResponseType tuntun_connection_dialog_show (TuntunConnectionDialog* self, GtkWidget* parent) {
	GtkResponseType response;
	g_return_val_if_fail (TUNTUN_IS_CONNECTION_DIALOG (self), 0);
	g_return_val_if_fail (GTK_IS_WIDGET (parent), 0);
	response = ((GtkResponseType) (gtk_dialog_run (self->priv->_window)));
	tuntun_connection_dialog_cleanup (self);
	gtk_widget_hide (GTK_WIDGET (self->priv->_window));
	return response;
}


static void tuntun_connection_dialog_on_button_connection_ok_clicked (TuntunConnectionDialog* self, GtkButton* sender) {
	g_return_if_fail (TUNTUN_IS_CONNECTION_DIALOG (self));
	g_return_if_fail (GTK_IS_BUTTON (sender));
	tuntun_connection_dialog_update_connection_info_object (self);
	gtk_dialog_response (self->priv->_window, ((gint) (GTK_RESPONSE_OK)));
}


static void tuntun_connection_dialog_on_button_connection_cancel_clicked (TuntunConnectionDialog* self, GtkButton* sender) {
	g_return_if_fail (TUNTUN_IS_CONNECTION_DIALOG (self));
	g_return_if_fail (GTK_IS_BUTTON (sender));
	gtk_dialog_response (self->priv->_window, ((gint) (GTK_RESPONSE_CANCEL)));
}


static void tuntun_connection_dialog_update_text_entries (TuntunConnectionDialog* self) {
	char* _tmp0;
	g_return_if_fail (TUNTUN_IS_CONNECTION_DIALOG (self));
	tuntun_connection_dialog_set_text_if_null (self, self->priv->_name, tuntun_connection_info_get_name (self->priv->_connection_info));
	tuntun_connection_dialog_set_text_if_null (self, self->priv->_address, tuntun_connection_info_get_address (self->priv->_connection_info));
	_tmp0 = NULL;
	tuntun_connection_dialog_set_text_if_null (self, self->priv->_port, (_tmp0 = g_strdup_printf ("%i", tuntun_connection_info_get_port (self->priv->_connection_info))));
	_tmp0 = (g_free (_tmp0), NULL);
}


static void tuntun_connection_dialog_update_connection_info_object (TuntunConnectionDialog* self) {
	g_return_if_fail (TUNTUN_IS_CONNECTION_DIALOG (self));
	tuntun_connection_info_set_name (self->priv->_connection_info, gtk_entry_get_text (self->priv->_name));
	tuntun_connection_info_set_address (self->priv->_connection_info, gtk_entry_get_text (self->priv->_address));
	tuntun_connection_info_set_port (self->priv->_connection_info, atoi (gtk_entry_get_text (self->priv->_port)));
}


static void tuntun_connection_dialog_set_text_if_null (TuntunConnectionDialog* self, GtkEntry* entry, const char* text) {
	g_return_if_fail (TUNTUN_IS_CONNECTION_DIALOG (self));
	g_return_if_fail (GTK_IS_ENTRY (entry));
	g_return_if_fail (text != NULL);
	if (text != NULL) {
		gtk_entry_set_text (entry, text);
	} else {
		gtk_entry_set_text (entry, "");
	}
}


static void tuntun_connection_dialog_set_connection_info (TuntunConnectionDialog* self, TuntunConnectionInfo* value) {
	TuntunConnectionInfo* _tmp1;
	TuntunConnectionInfo* _tmp0;
	g_return_if_fail (TUNTUN_IS_CONNECTION_DIALOG (self));
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->_connection_info = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL ? NULL : g_object_ref (_tmp0))), (self->priv->_connection_info == NULL ? NULL : (self->priv->_connection_info = (g_object_unref (self->priv->_connection_info), NULL))), _tmp1);
	g_object_notify (((GObject *) (self)), "connection-info");
}


static GObject * tuntun_connection_dialog_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	TuntunConnectionDialogClass * klass;
	GObjectClass * parent_class;
	TuntunConnectionDialog * self;
	klass = TUNTUN_CONNECTION_DIALOG_CLASS (g_type_class_peek (TUNTUN_TYPE_CONNECTION_DIALOG));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = TUNTUN_CONNECTION_DIALOG (obj);
	{
		tuntun_connection_dialog_initialize (self);
	}
	return obj;
}


static void tuntun_connection_dialog_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	TuntunConnectionDialog * self;
	self = TUNTUN_CONNECTION_DIALOG (object);
	switch (property_id) {
		case TUNTUN_CONNECTION_DIALOG_CONNECTION_INFO:
		tuntun_connection_dialog_set_connection_info (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void tuntun_connection_dialog_class_init (TuntunConnectionDialogClass * klass) {
	tuntun_connection_dialog_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (TuntunConnectionDialogPrivate));
	G_OBJECT_CLASS (klass)->set_property = tuntun_connection_dialog_set_property;
	G_OBJECT_CLASS (klass)->constructor = tuntun_connection_dialog_constructor;
	G_OBJECT_CLASS (klass)->dispose = tuntun_connection_dialog_dispose;
	g_object_class_install_property (G_OBJECT_CLASS (klass), TUNTUN_CONNECTION_DIALOG_CONNECTION_INFO, g_param_spec_object ("connection-info", "connection-info", "connection-info", TUNTUN_TYPE_CONNECTION_INFO, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_signal_new ("closed", TUNTUN_TYPE_CONNECTION_DIALOG, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}


static void tuntun_connection_dialog_instance_init (TuntunConnectionDialog * self) {
	self->priv = TUNTUN_CONNECTION_DIALOG_GET_PRIVATE (self);
	self->priv->_window = NULL;
	self->priv->_name = NULL;
	self->priv->_address = NULL;
	self->priv->_port = NULL;
	self->priv->_connection_info = NULL;
}


static void tuntun_connection_dialog_dispose (GObject * obj) {
	TuntunConnectionDialog * self;
	self = TUNTUN_CONNECTION_DIALOG (obj);
	(self->priv->_store == NULL ? NULL : (self->priv->_store = (g_object_unref (self->priv->_store), NULL)));
	(self->priv->_connection_info == NULL ? NULL : (self->priv->_connection_info = (g_object_unref (self->priv->_connection_info), NULL)));
	G_OBJECT_CLASS (tuntun_connection_dialog_parent_class)->dispose (obj);
}


GType tuntun_connection_dialog_get_type (void) {
	static volatile gsize tuntun_connection_dialog_type_id = 0;
	if (g_once_init_enter (&tuntun_connection_dialog_type_id)) {
		GType tuntun_connection_dialog_type_id_temp;
		static const GTypeInfo g_define_type_info = { sizeof (TuntunConnectionDialogClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) tuntun_connection_dialog_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (TuntunConnectionDialog), 0, (GInstanceInitFunc) tuntun_connection_dialog_instance_init };
		tuntun_connection_dialog_type_id_temp = g_type_register_static (G_TYPE_OBJECT, "TuntunConnectionDialog", &g_define_type_info, 0);
		g_once_init_leave (&tuntun_connection_dialog_type_id, tuntun_connection_dialog_type_id_temp);
	}
	return tuntun_connection_dialog_type_id;
}




