/*
 *  tuntun-connection-info.vala is a part of Tuntun
 *
 *  Tuntun: a simple applet to manage OpenVPN connections
 *
 *  Copyright (C) 2008  Andrea Del Signore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
 *
 *  Author:
 *     Andrea Del Signore <sejerpz@tin.it>       
 */

#include "tuntun-connection-info.h"
#include <glib/gi18n-lib.h>




struct _TuntunConnectionInfoPrivate {
	char* _name;
	char* _address;
	gint _port;
	char* _assigned_ip;
	gboolean _quick_connect;
};

#define TUNTUN_CONNECTION_INFO_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TUNTUN_TYPE_CONNECTION_INFO, TuntunConnectionInfoPrivate))
enum  {
	TUNTUN_CONNECTION_INFO_DUMMY_PROPERTY,
	TUNTUN_CONNECTION_INFO_NAME,
	TUNTUN_CONNECTION_INFO_ADDRESS,
	TUNTUN_CONNECTION_INFO_PORT,
	TUNTUN_CONNECTION_INFO_ASSIGNED_IP,
	TUNTUN_CONNECTION_INFO_QUICK_CONNECT
};
static GObject * tuntun_connection_info_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer tuntun_connection_info_parent_class = NULL;
static void tuntun_connection_info_dispose (GObject * obj);



TuntunConnectionInfo* tuntun_connection_info_new (void) {
	TuntunConnectionInfo * self;
	self = g_object_newv (TUNTUN_TYPE_CONNECTION_INFO, 0, NULL);
	return self;
}


const char* tuntun_connection_info_get_name (TuntunConnectionInfo* self) {
	g_return_val_if_fail (TUNTUN_IS_CONNECTION_INFO (self), NULL);
	return self->priv->_name;
}


void tuntun_connection_info_set_name (TuntunConnectionInfo* self, const char* value) {
	char* _tmp2;
	const char* _tmp1;
	g_return_if_fail (TUNTUN_IS_CONNECTION_INFO (self));
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_name = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_strdup (_tmp1))), (self->priv->_name = (g_free (self->priv->_name), NULL)), _tmp2);
	g_object_notify (((GObject *) (self)), "name");
}


const char* tuntun_connection_info_get_address (TuntunConnectionInfo* self) {
	g_return_val_if_fail (TUNTUN_IS_CONNECTION_INFO (self), NULL);
	return self->priv->_address;
}


void tuntun_connection_info_set_address (TuntunConnectionInfo* self, const char* value) {
	char* _tmp2;
	const char* _tmp1;
	g_return_if_fail (TUNTUN_IS_CONNECTION_INFO (self));
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_address = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_strdup (_tmp1))), (self->priv->_address = (g_free (self->priv->_address), NULL)), _tmp2);
	g_object_notify (((GObject *) (self)), "address");
}


gint tuntun_connection_info_get_port (TuntunConnectionInfo* self) {
	g_return_val_if_fail (TUNTUN_IS_CONNECTION_INFO (self), 0);
	return self->priv->_port;
}


void tuntun_connection_info_set_port (TuntunConnectionInfo* self, gint value) {
	g_return_if_fail (TUNTUN_IS_CONNECTION_INFO (self));
	self->priv->_port = value;
	g_object_notify (((GObject *) (self)), "port");
}


const char* tuntun_connection_info_get_assigned_ip (TuntunConnectionInfo* self) {
	g_return_val_if_fail (TUNTUN_IS_CONNECTION_INFO (self), NULL);
	return self->priv->_assigned_ip;
}


void tuntun_connection_info_set_assigned_ip (TuntunConnectionInfo* self, const char* value) {
	char* _tmp2;
	const char* _tmp1;
	g_return_if_fail (TUNTUN_IS_CONNECTION_INFO (self));
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_assigned_ip = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_strdup (_tmp1))), (self->priv->_assigned_ip = (g_free (self->priv->_assigned_ip), NULL)), _tmp2);
	g_object_notify (((GObject *) (self)), "assigned-ip");
}


gboolean tuntun_connection_info_get_quick_connect (TuntunConnectionInfo* self) {
	g_return_val_if_fail (TUNTUN_IS_CONNECTION_INFO (self), FALSE);
	return self->priv->_quick_connect;
}


void tuntun_connection_info_set_quick_connect (TuntunConnectionInfo* self, gboolean value) {
	g_return_if_fail (TUNTUN_IS_CONNECTION_INFO (self));
	self->priv->_quick_connect = value;
	g_object_notify (((GObject *) (self)), "quick-connect");
}


static GObject * tuntun_connection_info_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	TuntunConnectionInfoClass * klass;
	GObjectClass * parent_class;
	TuntunConnectionInfo * self;
	klass = TUNTUN_CONNECTION_INFO_CLASS (g_type_class_peek (TUNTUN_TYPE_CONNECTION_INFO));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = TUNTUN_CONNECTION_INFO (obj);
	{
		char* _tmp0;
		char* _tmp1;
		char* _tmp3;
		const char* _tmp2;
		_tmp0 = NULL;
		self->priv->_name = (_tmp0 = g_strdup (""), (self->priv->_name = (g_free (self->priv->_name), NULL)), _tmp0);
		_tmp1 = NULL;
		self->priv->_address = (_tmp1 = g_strdup (""), (self->priv->_address = (g_free (self->priv->_address), NULL)), _tmp1);
		self->priv->_port = 0;
		_tmp3 = NULL;
		_tmp2 = NULL;
		self->priv->_assigned_ip = (_tmp3 = (_tmp2 = _ ("none"), (_tmp2 == NULL ? NULL : g_strdup (_tmp2))), (self->priv->_assigned_ip = (g_free (self->priv->_assigned_ip), NULL)), _tmp3);
		self->priv->_quick_connect = TRUE;
	}
	return obj;
}


static void tuntun_connection_info_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	TuntunConnectionInfo * self;
	self = TUNTUN_CONNECTION_INFO (object);
	switch (property_id) {
		case TUNTUN_CONNECTION_INFO_NAME:
		g_value_set_string (value, tuntun_connection_info_get_name (self));
		break;
		case TUNTUN_CONNECTION_INFO_ADDRESS:
		g_value_set_string (value, tuntun_connection_info_get_address (self));
		break;
		case TUNTUN_CONNECTION_INFO_PORT:
		g_value_set_int (value, tuntun_connection_info_get_port (self));
		break;
		case TUNTUN_CONNECTION_INFO_ASSIGNED_IP:
		g_value_set_string (value, tuntun_connection_info_get_assigned_ip (self));
		break;
		case TUNTUN_CONNECTION_INFO_QUICK_CONNECT:
		g_value_set_boolean (value, tuntun_connection_info_get_quick_connect (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void tuntun_connection_info_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	TuntunConnectionInfo * self;
	self = TUNTUN_CONNECTION_INFO (object);
	switch (property_id) {
		case TUNTUN_CONNECTION_INFO_NAME:
		tuntun_connection_info_set_name (self, g_value_get_string (value));
		break;
		case TUNTUN_CONNECTION_INFO_ADDRESS:
		tuntun_connection_info_set_address (self, g_value_get_string (value));
		break;
		case TUNTUN_CONNECTION_INFO_PORT:
		tuntun_connection_info_set_port (self, g_value_get_int (value));
		break;
		case TUNTUN_CONNECTION_INFO_ASSIGNED_IP:
		tuntun_connection_info_set_assigned_ip (self, g_value_get_string (value));
		break;
		case TUNTUN_CONNECTION_INFO_QUICK_CONNECT:
		tuntun_connection_info_set_quick_connect (self, g_value_get_boolean (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void tuntun_connection_info_class_init (TuntunConnectionInfoClass * klass) {
	tuntun_connection_info_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (TuntunConnectionInfoPrivate));
	G_OBJECT_CLASS (klass)->get_property = tuntun_connection_info_get_property;
	G_OBJECT_CLASS (klass)->set_property = tuntun_connection_info_set_property;
	G_OBJECT_CLASS (klass)->constructor = tuntun_connection_info_constructor;
	G_OBJECT_CLASS (klass)->dispose = tuntun_connection_info_dispose;
	g_object_class_install_property (G_OBJECT_CLASS (klass), TUNTUN_CONNECTION_INFO_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), TUNTUN_CONNECTION_INFO_ADDRESS, g_param_spec_string ("address", "address", "address", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), TUNTUN_CONNECTION_INFO_PORT, g_param_spec_int ("port", "port", "port", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), TUNTUN_CONNECTION_INFO_ASSIGNED_IP, g_param_spec_string ("assigned-ip", "assigned-ip", "assigned-ip", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), TUNTUN_CONNECTION_INFO_QUICK_CONNECT, g_param_spec_boolean ("quick-connect", "quick-connect", "quick-connect", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void tuntun_connection_info_instance_init (TuntunConnectionInfo * self) {
	self->priv = TUNTUN_CONNECTION_INFO_GET_PRIVATE (self);
}


static void tuntun_connection_info_dispose (GObject * obj) {
	TuntunConnectionInfo * self;
	self = TUNTUN_CONNECTION_INFO (obj);
	self->priv->_name = (g_free (self->priv->_name), NULL);
	self->priv->_address = (g_free (self->priv->_address), NULL);
	self->priv->_assigned_ip = (g_free (self->priv->_assigned_ip), NULL);
	G_OBJECT_CLASS (tuntun_connection_info_parent_class)->dispose (obj);
}


GType tuntun_connection_info_get_type (void) {
	static volatile gsize tuntun_connection_info_type_id = 0;
	if (g_once_init_enter (&tuntun_connection_info_type_id)) {
		GType tuntun_connection_info_type_id_temp;
		static const GTypeInfo g_define_type_info = { sizeof (TuntunConnectionInfoClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) tuntun_connection_info_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (TuntunConnectionInfo), 0, (GInstanceInitFunc) tuntun_connection_info_instance_init };
		tuntun_connection_info_type_id_temp = g_type_register_static (G_TYPE_OBJECT, "TuntunConnectionInfo", &g_define_type_info, 0);
		g_once_init_leave (&tuntun_connection_info_type_id, tuntun_connection_info_type_id_temp);
	}
	return tuntun_connection_info_type_id;
}




