/*
 *  tuntun-connection-info.vala is a part of Tuntun
 *
 *  Tuntun: a simple applet to manage OpenVPN connections
 *
 *  Copyright (C) 2008  Andrea Del Signore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
 *
 *  Author:
 *     Andrea Del Signore <sejerpz@tin.it>       
 */

#ifndef __TUNTUN_CONNECTION_INFO_H__
#define __TUNTUN_CONNECTION_INFO_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>

G_BEGIN_DECLS


#define TUNTUN_TYPE_CONNECTION_INFO (tuntun_connection_info_get_type ())
#define TUNTUN_CONNECTION_INFO(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TUNTUN_TYPE_CONNECTION_INFO, TuntunConnectionInfo))
#define TUNTUN_CONNECTION_INFO_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TUNTUN_TYPE_CONNECTION_INFO, TuntunConnectionInfoClass))
#define TUNTUN_IS_CONNECTION_INFO(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TUNTUN_TYPE_CONNECTION_INFO))
#define TUNTUN_IS_CONNECTION_INFO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TUNTUN_TYPE_CONNECTION_INFO))
#define TUNTUN_CONNECTION_INFO_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TUNTUN_TYPE_CONNECTION_INFO, TuntunConnectionInfoClass))

typedef struct _TuntunConnectionInfo TuntunConnectionInfo;
typedef struct _TuntunConnectionInfoClass TuntunConnectionInfoClass;
typedef struct _TuntunConnectionInfoPrivate TuntunConnectionInfoPrivate;

struct _TuntunConnectionInfo {
	GObject parent_instance;
	TuntunConnectionInfoPrivate * priv;
};

struct _TuntunConnectionInfoClass {
	GObjectClass parent_class;
};


TuntunConnectionInfo* tuntun_connection_info_new (void);
const char* tuntun_connection_info_get_name (TuntunConnectionInfo* self);
void tuntun_connection_info_set_name (TuntunConnectionInfo* self, const char* value);
const char* tuntun_connection_info_get_address (TuntunConnectionInfo* self);
void tuntun_connection_info_set_address (TuntunConnectionInfo* self, const char* value);
gint tuntun_connection_info_get_port (TuntunConnectionInfo* self);
void tuntun_connection_info_set_port (TuntunConnectionInfo* self, gint value);
const char* tuntun_connection_info_get_assigned_ip (TuntunConnectionInfo* self);
void tuntun_connection_info_set_assigned_ip (TuntunConnectionInfo* self, const char* value);
gboolean tuntun_connection_info_get_quick_connect (TuntunConnectionInfo* self);
void tuntun_connection_info_set_quick_connect (TuntunConnectionInfo* self, gboolean value);
GType tuntun_connection_info_get_type (void);


G_END_DECLS

#endif
