
#ifndef __TUNTUN_CONNECTION_H__
#define __TUNTUN_CONNECTION_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include "tuntun-connection-info.h"

G_BEGIN_DECLS


#define TUNTUN_TYPE_CONNECTION_STATES (tuntun_connection_states_get_type ())

#define TUNTUN_TYPE_AUTHENTICATION_MODES (tuntun_authentication_modes_get_type ())

#define TUNTUN_TYPE_CONNECTION (tuntun_connection_get_type ())
#define TUNTUN_CONNECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TUNTUN_TYPE_CONNECTION, TuntunConnection))
#define TUNTUN_CONNECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TUNTUN_TYPE_CONNECTION, TuntunConnectionClass))
#define TUNTUN_IS_CONNECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TUNTUN_TYPE_CONNECTION))
#define TUNTUN_IS_CONNECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TUNTUN_TYPE_CONNECTION))
#define TUNTUN_CONNECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TUNTUN_TYPE_CONNECTION, TuntunConnectionClass))

typedef struct _TuntunConnection TuntunConnection;
typedef struct _TuntunConnectionClass TuntunConnectionClass;
typedef struct _TuntunConnectionPrivate TuntunConnectionPrivate;

typedef enum  {
	TUNTUN_CONNECTION_STATES_UNKNOWN = 0,
	TUNTUN_CONNECTION_STATES_ERROR = 1,
	TUNTUN_CONNECTION_STATES_CONNECTED = 2,
	TUNTUN_CONNECTION_STATES_CONNECTING = 3,
	TUNTUN_CONNECTION_STATES_DISCONNECTED = 4,
	TUNTUN_CONNECTION_STATES_DISCONNECTING = 5
} TuntunConnectionStates;

typedef enum  {
	TUNTUN_AUTHENTICATION_MODES_PASSWORD_ONLY,
	TUNTUN_AUTHENTICATION_MODES_USERNAME_PASSWORD
} TuntunAuthenticationModes;

struct _TuntunConnection {
	GObject parent_instance;
	TuntunConnectionPrivate * priv;
};

struct _TuntunConnectionClass {
	GObjectClass parent_class;
};


GType tuntun_connection_states_get_type (void);
GType tuntun_authentication_modes_get_type (void);
TuntunConnection* tuntun_connection_new (TuntunConnectionInfo* info);
gboolean tuntun_connection_initialize (TuntunConnection* self);
gboolean tuntun_connection_reinitialize (TuntunConnection* self);
void tuntun_connection_send (TuntunConnection* self, const char* data);
void tuntun_connection_connect (TuntunConnection* self);
void tuntun_connection_disconnect (TuntunConnection* self);
void tuntun_connection_authenticate (TuntunConnection* self, const char* type, const char* username, const char* password);
void tuntun_connection_cancel_authentication (TuntunConnection* self, const char* type, const char* username);
TuntunConnectionInfo* tuntun_connection_get_info (TuntunConnection* self);
TuntunConnectionStates tuntun_connection_get_control_channel_status (TuntunConnection* self);
void tuntun_connection_set_control_channel_status (TuntunConnection* self, TuntunConnectionStates value);
TuntunConnectionStates tuntun_connection_get_status (TuntunConnection* self);
void tuntun_connection_set_status (TuntunConnection* self, TuntunConnectionStates value);
GType tuntun_connection_get_type (void);


G_END_DECLS

#endif
