/*
 *  tuntun-connections-dialog.vala is a part of Tuntun
 *
 *  Tuntun: a simple applet to manage OpenVPN connections
 *
 *  Copyright (C) 2008  Andrea Del Signore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
 *
 *  Author:
 *     Andrea Del Signore <sejerpz@tin.it>       
 */

#include "tuntun-connections-dialog.h"
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <stdlib.h>
#include <string.h>
#include <glib/gi18n-lib.h>
#include <gdk/gdk.h>
#include "tuntun-utils.h"
#include "tuntun-constants.h"
#include "tuntun-connections.h"
#include "tuntun-connection.h"
#include "tuntun-connection-info.h"
#include "tuntun-connection-dialog.h"


#define TUNTUN_TYPE_COLUMNS (tuntun_columns_get_type ())

typedef enum  {
	TUNTUN_COLUMNS_DOUBLE_CLICK_CONNECT,
	TUNTUN_COLUMNS_NAME,
	TUNTUN_COLUMNS_ADDRESS,
	TUNTUN_COLUMNS_PORT,
	TUNTUN_COLUMNS_STATUS,
	TUNTUN_COLUMNS_ASSIGNED_IP,
	TUNTUN_COLUMNS_CONNECTION,
	TUNTUN_COLUMNS_COUNT
} TuntunColumns;



GType tuntun_columns_get_type (void);
struct _TuntunConnectionsDialogPrivate {
	GtkWindow* _window;
	GtkTreeView* _treeview;
	GtkButton* button_conn_remove;
	GtkButton* button_conn_modify;
	GtkListStore* _store;
	TuntunTuntun* _tuntun;
	GdkPixbuf** _pixbufs;
	gint _pixbufs_length1;
	gboolean is_dirty;
};

#define TUNTUN_CONNECTIONS_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TUNTUN_TYPE_CONNECTIONS_DIALOG, TuntunConnectionsDialogPrivate))
enum  {
	TUNTUN_CONNECTIONS_DIALOG_DUMMY_PROPERTY,
	TUNTUN_CONNECTIONS_DIALOG_TUNTUN
};
static gboolean _tuntun_connections_dialog_on_window_delete_gtk_widget_delete_event (GtkWindow* _sender, GdkEvent* event, gpointer self);
static void _tuntun_connections_dialog_on_treeview_selection_changed_gtk_tree_selection_changed (GtkTreeSelection* _sender, gpointer self);
static void _tuntun_connections_dialog_on_double_click_connect_toggled_gtk_cell_renderer_toggle_toggled (GtkCellRendererToggle* _sender, const char* path, gpointer self);
static void _tuntun_connections_dialog_on_button_connection_add_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self);
static void _tuntun_connections_dialog_on_button_connection_remove_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self);
static void _tuntun_connections_dialog_on_button_connection_modify_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self);
static void _tuntun_connections_dialog_on_button_close_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self);
static void _tuntun_connections_dialog_on_connection_added_tuntun_connections_connection_added (TuntunConnections* _sender, TuntunConnection* connection, gpointer self);
static void _tuntun_connections_dialog_on_connection_removed_tuntun_connections_connection_removed (TuntunConnections* _sender, TuntunConnection* connection, gpointer self);
static void tuntun_connections_dialog_initialize (TuntunConnectionsDialog* self);
static void tuntun_connections_dialog_on_double_click_connect_toggled (TuntunConnectionsDialog* self, GtkCellRendererToggle* cell_renderer, const char* path);
static void tuntun_connections_dialog_cleanup_and_close (TuntunConnectionsDialog* self);
static void tuntun_connections_dialog_on_connection_added (TuntunConnectionsDialog* self, TuntunConnections* connections, TuntunConnection* connection);
static void tuntun_connections_dialog_on_connection_removed (TuntunConnectionsDialog* self, TuntunConnections* connections, TuntunConnection* connection);
static void tuntun_connections_dialog_on_treeview_selection_changed (TuntunConnectionsDialog* self, GtkTreeSelection* sender);
static gboolean tuntun_connections_dialog_on_window_delete (TuntunConnectionsDialog* self, GtkWindow* sender, GdkEvent* event);
static void tuntun_connections_dialog_on_button_connection_add_clicked (TuntunConnectionsDialog* self, GtkButton* sender);
static void tuntun_connections_dialog_on_button_connection_remove_clicked (TuntunConnectionsDialog* self, GtkButton* sender);
static void tuntun_connections_dialog_on_button_connection_modify_clicked (TuntunConnectionsDialog* self, GtkButton* sender);
static void tuntun_connections_dialog_on_button_close_clicked (TuntunConnectionsDialog* self, GtkButton* sender);
static void tuntun_connections_dialog_store_add_item (TuntunConnectionsDialog* self, TuntunConnection* connection);
static void tuntun_connections_dialog_store_modify_item (TuntunConnectionsDialog* self, GtkTreeIter* iter, TuntunConnection* connection);
static void tuntun_connections_dialog_store_remove_item (TuntunConnectionsDialog* self, TuntunConnection* connection);
static gboolean tuntun_connections_dialog_store_find (TuntunConnectionsDialog* self, TuntunConnection* connection, GtkTreeIter* iter);
static void tuntun_connections_dialog_set_tuntun (TuntunConnectionsDialog* self, TuntunTuntun* value);
static TuntunConnection* tuntun_connections_dialog_get_selected_connection (TuntunConnectionsDialog* self);
static GObject * tuntun_connections_dialog_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer tuntun_connections_dialog_parent_class = NULL;
static void tuntun_connections_dialog_dispose (GObject * obj);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static int _vala_strcmp0 (const char * str1, const char * str2);




GType tuntun_columns_get_type (void) {
	static GType tuntun_columns_type_id = 0;
	if (G_UNLIKELY (tuntun_columns_type_id == 0)) {
		static const GEnumValue values[] = {{TUNTUN_COLUMNS_DOUBLE_CLICK_CONNECT, "TUNTUN_COLUMNS_DOUBLE_CLICK_CONNECT", "double-click-connect"}, {TUNTUN_COLUMNS_NAME, "TUNTUN_COLUMNS_NAME", "name"}, {TUNTUN_COLUMNS_ADDRESS, "TUNTUN_COLUMNS_ADDRESS", "address"}, {TUNTUN_COLUMNS_PORT, "TUNTUN_COLUMNS_PORT", "port"}, {TUNTUN_COLUMNS_STATUS, "TUNTUN_COLUMNS_STATUS", "status"}, {TUNTUN_COLUMNS_ASSIGNED_IP, "TUNTUN_COLUMNS_ASSIGNED_IP", "assigned-ip"}, {TUNTUN_COLUMNS_CONNECTION, "TUNTUN_COLUMNS_CONNECTION", "connection"}, {TUNTUN_COLUMNS_COUNT, "TUNTUN_COLUMNS_COUNT", "count"}, {0, NULL, NULL}};
		tuntun_columns_type_id = g_enum_register_static ("TuntunColumns", values);
	}
	return tuntun_columns_type_id;
}


static gboolean _tuntun_connections_dialog_on_window_delete_gtk_widget_delete_event (GtkWindow* _sender, GdkEvent* event, gpointer self) {
	return tuntun_connections_dialog_on_window_delete (self, _sender, event);
}


static void _tuntun_connections_dialog_on_treeview_selection_changed_gtk_tree_selection_changed (GtkTreeSelection* _sender, gpointer self) {
	tuntun_connections_dialog_on_treeview_selection_changed (self, _sender);
}


static void _tuntun_connections_dialog_on_double_click_connect_toggled_gtk_cell_renderer_toggle_toggled (GtkCellRendererToggle* _sender, const char* path, gpointer self) {
	tuntun_connections_dialog_on_double_click_connect_toggled (self, _sender, path);
}


static void _tuntun_connections_dialog_on_button_connection_add_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self) {
	tuntun_connections_dialog_on_button_connection_add_clicked (self, _sender);
}


static void _tuntun_connections_dialog_on_button_connection_remove_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self) {
	tuntun_connections_dialog_on_button_connection_remove_clicked (self, _sender);
}


static void _tuntun_connections_dialog_on_button_connection_modify_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self) {
	tuntun_connections_dialog_on_button_connection_modify_clicked (self, _sender);
}


static void _tuntun_connections_dialog_on_button_close_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self) {
	tuntun_connections_dialog_on_button_close_clicked (self, _sender);
}


static void _tuntun_connections_dialog_on_connection_added_tuntun_connections_connection_added (TuntunConnections* _sender, TuntunConnection* connection, gpointer self) {
	tuntun_connections_dialog_on_connection_added (self, _sender, connection);
}


static void _tuntun_connections_dialog_on_connection_removed_tuntun_connections_connection_removed (TuntunConnections* _sender, TuntunConnection* connection, gpointer self) {
	tuntun_connections_dialog_on_connection_removed (self, _sender, connection);
}


static void tuntun_connections_dialog_initialize (TuntunConnectionsDialog* self) {
	GError * inner_error;
	GtkBuilder* _tmp0;
	GtkBuilder* builder;
	GdkPixbuf** _tmp1;
	GtkCellRenderer* renderer;
	GtkTreeViewColumn* column;
	GtkTreeSelection* _tmp8;
	GtkTreeSelection* selection;
	GtkCellRenderer* _tmp9;
	GtkTreeViewColumn* _tmp10;
	GtkCellRenderer* _tmp11;
	GtkCellRendererMode _tmp12;
	GtkTreeViewColumn* _tmp13;
	GtkCellRenderer* _tmp14;
	GtkTreeViewColumn* _tmp15;
	GtkCellRenderer* _tmp16;
	GtkTreeViewColumn* _tmp17;
	GtkCellRenderer* _tmp18;
	GtkTreeViewColumn* _tmp19;
	GtkCellRenderer* _tmp20;
	GtkTreeViewColumn* _tmp21;
	GtkButton* _tmp22;
	GtkButton* button;
	GtkButton* _tmp24;
	GtkButton* _tmp23;
	GtkListStore* _tmp25;
	g_return_if_fail (TUNTUN_IS_CONNECTIONS_DIALOG (self));
	inner_error = NULL;
	_tmp0 = NULL;
	builder = (_tmp0 = tuntun_utils_get_ui (), (_tmp0 == NULL ? NULL : g_object_ref (_tmp0)));
	self->priv->_treeview = GTK_TREE_VIEW (gtk_builder_get_object (builder, "treeview_connections"));
	g_assert (self->priv->_treeview != NULL);
	self->priv->_window = GTK_WINDOW (gtk_builder_get_object (builder, "window_connections"));
	g_assert (self->priv->_window != NULL);
	/* one time initialization */
	g_signal_connect_object (GTK_WIDGET (self->priv->_window), "delete-event", ((GCallback) (_tuntun_connections_dialog_on_window_delete_gtk_widget_delete_event)), self, 0);
	g_signal_connect_object (gtk_tree_view_get_selection (self->priv->_treeview), "changed", ((GCallback) (_tuntun_connections_dialog_on_treeview_selection_changed_gtk_tree_selection_changed)), self, 0);
	/* connection status pixbufs */
	_tmp1 = NULL;
	self->priv->_pixbufs = (_tmp1 = g_new0 (GdkPixbuf*, 3 + 1), (self->priv->_pixbufs = (_vala_array_free (self->priv->_pixbufs, self->priv->_pixbufs_length1, ((GDestroyNotify) (g_object_unref))), NULL)), self->priv->_pixbufs_length1 = 3, _tmp1);
	{
		GdkPixbuf* _tmp3;
		char* _tmp2;
		GdkPixbuf* _tmp5;
		char* _tmp4;
		GdkPixbuf* _tmp7;
		char* _tmp6;
		_tmp3 = NULL;
		_tmp2 = NULL;
		self->priv->_pixbufs[0] = (_tmp3 = gdk_pixbuf_new_from_file ((_tmp2 = tuntun_utils_get_image_path (TUNTUN_CONSTANTS_IMAGES_CONNECTION_STATUS_UNKNOWN)), &inner_error), (self->priv->_pixbufs[0] == NULL ? NULL : (self->priv->_pixbufs[0] = (g_object_unref (self->priv->_pixbufs[0]), NULL))), _tmp3);
		if (inner_error != NULL) {
			goto __catch7_g_error;
		}
		_tmp2 = (g_free (_tmp2), NULL);
		_tmp5 = NULL;
		_tmp4 = NULL;
		self->priv->_pixbufs[1] = (_tmp5 = gdk_pixbuf_new_from_file ((_tmp4 = tuntun_utils_get_image_path (TUNTUN_CONSTANTS_IMAGES_CONNECTION_STATUS_DISCONNECT)), &inner_error), (self->priv->_pixbufs[1] == NULL ? NULL : (self->priv->_pixbufs[1] = (g_object_unref (self->priv->_pixbufs[1]), NULL))), _tmp5);
		if (inner_error != NULL) {
			goto __catch7_g_error;
		}
		_tmp4 = (g_free (_tmp4), NULL);
		_tmp7 = NULL;
		_tmp6 = NULL;
		self->priv->_pixbufs[2] = (_tmp7 = gdk_pixbuf_new_from_file ((_tmp6 = tuntun_utils_get_image_path (TUNTUN_CONSTANTS_IMAGES_CONNECTION_STATUS_CONNECT)), &inner_error), (self->priv->_pixbufs[2] == NULL ? NULL : (self->priv->_pixbufs[2] = (g_object_unref (self->priv->_pixbufs[2]), NULL))), _tmp7);
		if (inner_error != NULL) {
			goto __catch7_g_error;
		}
		_tmp6 = (g_free (_tmp6), NULL);
	}
	goto __finally7;
	__catch7_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_warning ("tuntun-connections-dialog.vala:83: ConnectionsDialog.initialize error reading status images: %s", err->message);
			(err == NULL ? NULL : (err = (g_error_free (err), NULL)));
		}
	}
	__finally7:
	;
	renderer = NULL;
	column = NULL;
	_tmp8 = NULL;
	selection = (_tmp8 = gtk_tree_view_get_selection (self->priv->_treeview), (_tmp8 == NULL ? NULL : g_object_ref (_tmp8)));
	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
	/* list view columns */
	_tmp9 = NULL;
	renderer = (_tmp9 = GTK_CELL_RENDERER (g_object_ref_sink (((GtkCellRendererToggle*) (gtk_cell_renderer_toggle_new ())))), (renderer == NULL ? NULL : (renderer = (g_object_unref (renderer), NULL))), _tmp9);
	g_signal_connect_object ((GTK_CELL_RENDERER_TOGGLE (renderer)), "toggled", ((GCallback) (_tuntun_connections_dialog_on_double_click_connect_toggled_gtk_cell_renderer_toggle_toggled)), self, 0);
	_tmp10 = NULL;
	column = (_tmp10 = g_object_ref_sink (gtk_tree_view_column_new_with_attributes (_ ("Quick connect"), renderer, "active", TUNTUN_COLUMNS_DOUBLE_CLICK_CONNECT, NULL)), (column == NULL ? NULL : (column = (g_object_unref (column), NULL))), _tmp10);
	gtk_tree_view_column_set_alignment (column, ((float) (0.50)));
	gtk_tree_view_column_set_expand (column, FALSE);
	gtk_tree_view_append_column (self->priv->_treeview, column);
	_tmp11 = NULL;
	renderer = (_tmp11 = GTK_CELL_RENDERER (g_object_ref_sink (((GtkCellRendererText*) (gtk_cell_renderer_text_new ())))), (renderer == NULL ? NULL : (renderer = (g_object_unref (renderer), NULL))), _tmp11);
	g_object_set (renderer, "mode", GTK_CELL_RENDERER_MODE_EDITABLE, NULL);
	_tmp13 = NULL;
	column = (_tmp13 = g_object_ref_sink (gtk_tree_view_column_new_with_attributes (_ ("Name"), renderer, "text", TUNTUN_COLUMNS_NAME, NULL)), (column == NULL ? NULL : (column = (g_object_unref (column), NULL))), _tmp13);
	gtk_tree_view_column_set_expand (column, FALSE);
	gtk_tree_view_append_column (self->priv->_treeview, column);
	_tmp14 = NULL;
	renderer = (_tmp14 = GTK_CELL_RENDERER (g_object_ref_sink (((GtkCellRendererText*) (gtk_cell_renderer_text_new ())))), (renderer == NULL ? NULL : (renderer = (g_object_unref (renderer), NULL))), _tmp14);
	_tmp15 = NULL;
	column = (_tmp15 = g_object_ref_sink (gtk_tree_view_column_new_with_attributes (_ ("Host"), renderer, "text", TUNTUN_COLUMNS_ADDRESS, NULL)), (column == NULL ? NULL : (column = (g_object_unref (column), NULL))), _tmp15);
	gtk_tree_view_column_set_expand (column, TRUE);
	gtk_tree_view_append_column (self->priv->_treeview, column);
	_tmp16 = NULL;
	renderer = (_tmp16 = GTK_CELL_RENDERER (g_object_ref_sink (((GtkCellRendererText*) (gtk_cell_renderer_text_new ())))), (renderer == NULL ? NULL : (renderer = (g_object_unref (renderer), NULL))), _tmp16);
	_tmp17 = NULL;
	column = (_tmp17 = g_object_ref_sink (gtk_tree_view_column_new_with_attributes (_ ("Port"), renderer, "text", TUNTUN_COLUMNS_PORT, NULL)), (column == NULL ? NULL : (column = (g_object_unref (column), NULL))), _tmp17);
	gtk_tree_view_column_set_expand (column, FALSE);
	gtk_tree_view_append_column (self->priv->_treeview, column);
	_tmp18 = NULL;
	renderer = (_tmp18 = GTK_CELL_RENDERER (g_object_ref_sink (((GtkCellRendererPixbuf*) (gtk_cell_renderer_pixbuf_new ())))), (renderer == NULL ? NULL : (renderer = (g_object_unref (renderer), NULL))), _tmp18);
	_tmp19 = NULL;
	column = (_tmp19 = g_object_ref_sink (gtk_tree_view_column_new_with_attributes (_ ("Status"), renderer, "pixbuf", TUNTUN_COLUMNS_STATUS, NULL)), (column == NULL ? NULL : (column = (g_object_unref (column), NULL))), _tmp19);
	gtk_tree_view_column_set_alignment (column, ((float) (0.50)));
	gtk_tree_view_column_set_expand (column, FALSE);
	gtk_tree_view_append_column (self->priv->_treeview, column);
	_tmp20 = NULL;
	renderer = (_tmp20 = GTK_CELL_RENDERER (g_object_ref_sink (((GtkCellRendererText*) (gtk_cell_renderer_text_new ())))), (renderer == NULL ? NULL : (renderer = (g_object_unref (renderer), NULL))), _tmp20);
	_tmp21 = NULL;
	column = (_tmp21 = g_object_ref_sink (gtk_tree_view_column_new_with_attributes (_ ("Assigned IP"), renderer, "text", TUNTUN_COLUMNS_ASSIGNED_IP, NULL)), (column == NULL ? NULL : (column = (g_object_unref (column), NULL))), _tmp21);
	gtk_tree_view_column_set_expand (column, FALSE);
	gtk_tree_view_append_column (self->priv->_treeview, column);
	/* buttons events */
	_tmp22 = NULL;
	button = (_tmp22 = GTK_BUTTON (gtk_builder_get_object (builder, "button_connection_add")), (_tmp22 == NULL ? NULL : g_object_ref (_tmp22)));
	g_assert (button != NULL);
	g_signal_connect_object (button, "clicked", ((GCallback) (_tuntun_connections_dialog_on_button_connection_add_clicked_gtk_button_clicked)), self, 0);
	self->priv->button_conn_remove = GTK_BUTTON (gtk_builder_get_object (builder, "button_connection_remove"));
	g_assert (self->priv->button_conn_remove != NULL);
	g_signal_connect_object (self->priv->button_conn_remove, "clicked", ((GCallback) (_tuntun_connections_dialog_on_button_connection_remove_clicked_gtk_button_clicked)), self, 0);
	self->priv->button_conn_modify = GTK_BUTTON (gtk_builder_get_object (builder, "button_connection_modify"));
	g_assert (self->priv->button_conn_modify != NULL);
	g_signal_connect_object (self->priv->button_conn_modify, "clicked", ((GCallback) (_tuntun_connections_dialog_on_button_connection_modify_clicked_gtk_button_clicked)), self, 0);
	_tmp24 = NULL;
	_tmp23 = NULL;
	button = (_tmp24 = (_tmp23 = GTK_BUTTON (gtk_builder_get_object (builder, "button_close")), (_tmp23 == NULL ? NULL : g_object_ref (_tmp23))), (button == NULL ? NULL : (button = (g_object_unref (button), NULL))), _tmp24);
	g_assert (button != NULL);
	g_signal_connect_object (button, "clicked", ((GCallback) (_tuntun_connections_dialog_on_button_close_clicked_gtk_button_clicked)), self, 0);
	/* initialize connection list view */
	_tmp25 = NULL;
	self->priv->_store = (_tmp25 = gtk_list_store_new (((gint) (TUNTUN_COLUMNS_COUNT)), G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, GDK_TYPE_PIXBUF, G_TYPE_STRING, TUNTUN_TYPE_CONNECTION, NULL), (self->priv->_store == NULL ? NULL : (self->priv->_store = (g_object_unref (self->priv->_store), NULL))), _tmp25);
	gtk_tree_view_set_model (self->priv->_treeview, GTK_TREE_MODEL (self->priv->_store));
	{
		GList* connection_collection;
		GList* connection_it;
		connection_collection = tuntun_tuntun_get_connections (self->priv->_tuntun)->items;
		for (connection_it = connection_collection; connection_it != NULL; connection_it = connection_it->next) {
			TuntunConnection* _tmp26;
			TuntunConnection* connection;
			_tmp26 = NULL;
			connection = (_tmp26 = ((TuntunConnection*) (connection_it->data)), (_tmp26 == NULL ? NULL : g_object_ref (_tmp26)));
			{
				tuntun_connections_dialog_store_add_item (self, connection);
				(connection == NULL ? NULL : (connection = (g_object_unref (connection), NULL)));
			}
		}
	}
	g_signal_connect_object (tuntun_tuntun_get_connections (self->priv->_tuntun), "connection-added", ((GCallback) (_tuntun_connections_dialog_on_connection_added_tuntun_connections_connection_added)), self, 0);
	g_signal_connect_object (tuntun_tuntun_get_connections (self->priv->_tuntun), "connection-removed", ((GCallback) (_tuntun_connections_dialog_on_connection_removed_tuntun_connections_connection_removed)), self, 0);
	(builder == NULL ? NULL : (builder = (g_object_unref (builder), NULL)));
	(renderer == NULL ? NULL : (renderer = (g_object_unref (renderer), NULL)));
	(column == NULL ? NULL : (column = (g_object_unref (column), NULL)));
	(selection == NULL ? NULL : (selection = (g_object_unref (selection), NULL)));
	(button == NULL ? NULL : (button = (g_object_unref (button), NULL)));
}


static void tuntun_connections_dialog_on_double_click_connect_toggled (TuntunConnectionsDialog* self, GtkCellRendererToggle* cell_renderer, const char* path) {
	GtkTreeIter iter = {0};
	g_return_if_fail (TUNTUN_IS_CONNECTIONS_DIALOG (self));
	g_return_if_fail (GTK_IS_CELL_RENDERER_TOGGLE (cell_renderer));
	g_return_if_fail (path != NULL);
	if (gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (self->priv->_store), &iter, path)) {
		TuntunConnection* conn;
		conn = NULL;
		gtk_tree_model_get (GTK_TREE_MODEL (self->priv->_store), &iter, TUNTUN_COLUMNS_CONNECTION, &conn, -1, -1);
		tuntun_connection_info_set_quick_connect (tuntun_connection_get_info (conn), !tuntun_connection_info_get_quick_connect (tuntun_connection_get_info (conn)));
		tuntun_connections_dialog_store_modify_item (self, &iter, conn);
		self->priv->is_dirty = TRUE;
	}
}


static void tuntun_connections_dialog_cleanup_and_close (TuntunConnectionsDialog* self) {
	guint _tmp0;
	GtkBuilder* _tmp1;
	GtkBuilder* builder;
	GtkButton* _tmp2;
	GtkButton* button;
	guint _tmp3;
	guint _tmp4;
	guint _tmp5;
	GtkButton* _tmp7;
	GtkButton* _tmp6;
	guint _tmp8;
	guint _tmp13;
	guint _tmp14;
	g_return_if_fail (TUNTUN_IS_CONNECTIONS_DIALOG (self));
	g_signal_handlers_disconnect_matched (GTK_WIDGET (self->priv->_window), G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("delete-event", GTK_TYPE_WIDGET, &_tmp0, NULL, FALSE), _tmp0), 0, NULL, ((GCallback) (_tuntun_connections_dialog_on_window_delete_gtk_widget_delete_event)), self);
	gtk_widget_hide (GTK_WIDGET (self->priv->_window));
	_tmp1 = NULL;
	builder = (_tmp1 = tuntun_utils_get_ui (), (_tmp1 == NULL ? NULL : g_object_ref (_tmp1)));
	_tmp2 = NULL;
	button = (_tmp2 = GTK_BUTTON (gtk_builder_get_object (builder, "button_connection_add")), (_tmp2 == NULL ? NULL : g_object_ref (_tmp2)));
	g_signal_handlers_disconnect_matched (button, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("clicked", GTK_TYPE_BUTTON, &_tmp3, NULL, FALSE), _tmp3), 0, NULL, ((GCallback) (_tuntun_connections_dialog_on_button_connection_add_clicked_gtk_button_clicked)), self);
	g_signal_handlers_disconnect_matched (self->priv->button_conn_remove, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("clicked", GTK_TYPE_BUTTON, &_tmp4, NULL, FALSE), _tmp4), 0, NULL, ((GCallback) (_tuntun_connections_dialog_on_button_connection_remove_clicked_gtk_button_clicked)), self);
	g_signal_handlers_disconnect_matched (self->priv->button_conn_modify, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("clicked", GTK_TYPE_BUTTON, &_tmp5, NULL, FALSE), _tmp5), 0, NULL, ((GCallback) (_tuntun_connections_dialog_on_button_connection_modify_clicked_gtk_button_clicked)), self);
	_tmp7 = NULL;
	_tmp6 = NULL;
	button = (_tmp7 = (_tmp6 = GTK_BUTTON (gtk_builder_get_object (builder, "button_close")), (_tmp6 == NULL ? NULL : g_object_ref (_tmp6))), (button == NULL ? NULL : (button = (g_object_unref (button), NULL))), _tmp7);
	g_signal_handlers_disconnect_matched (button, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("clicked", GTK_TYPE_BUTTON, &_tmp8, NULL, FALSE), _tmp8), 0, NULL, ((GCallback) (_tuntun_connections_dialog_on_button_close_clicked_gtk_button_clicked)), self);
	if (self->priv->_treeview != NULL) {
		guint _tmp9;
		GtkTreeViewColumn* _tmp10;
		GtkTreeViewColumn* column;
		g_signal_handlers_disconnect_matched (gtk_tree_view_get_selection (self->priv->_treeview), G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("changed", GTK_TYPE_TREE_SELECTION, &_tmp9, NULL, FALSE), _tmp9), 0, NULL, ((GCallback) (_tuntun_connections_dialog_on_treeview_selection_changed_gtk_tree_selection_changed)), self);
		_tmp10 = NULL;
		column = (_tmp10 = gtk_tree_view_get_column (self->priv->_treeview, 0), (_tmp10 == NULL ? NULL : g_object_ref (_tmp10)));
		while (column != NULL) {
			GtkTreeViewColumn* _tmp12;
			GtkTreeViewColumn* _tmp11;
			gtk_tree_view_remove_column (self->priv->_treeview, column);
			_tmp12 = NULL;
			_tmp11 = NULL;
			column = (_tmp12 = (_tmp11 = gtk_tree_view_get_column (self->priv->_treeview, 0), (_tmp11 == NULL ? NULL : g_object_ref (_tmp11))), (column == NULL ? NULL : (column = (g_object_unref (column), NULL))), _tmp12);
		}
		(column == NULL ? NULL : (column = (g_object_unref (column), NULL)));
	}
	g_signal_emit_by_name (G_OBJECT (self), "closed");
	g_signal_handlers_disconnect_matched (tuntun_tuntun_get_connections (self->priv->_tuntun), G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("connection-added", TUNTUN_TYPE_CONNECTIONS, &_tmp13, NULL, FALSE), _tmp13), 0, NULL, ((GCallback) (_tuntun_connections_dialog_on_connection_added_tuntun_connections_connection_added)), self);
	g_signal_handlers_disconnect_matched (tuntun_tuntun_get_connections (self->priv->_tuntun), G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("connection-removed", TUNTUN_TYPE_CONNECTIONS, &_tmp14, NULL, FALSE), _tmp14), 0, NULL, ((GCallback) (_tuntun_connections_dialog_on_connection_removed_tuntun_connections_connection_removed)), self);
	(builder == NULL ? NULL : (builder = (g_object_unref (builder), NULL)));
	(button == NULL ? NULL : (button = (g_object_unref (button), NULL)));
}


TuntunConnectionsDialog* tuntun_connections_dialog_new (TuntunTuntun* tuntun) {
	GParameter * __params;
	GParameter * __params_it;
	TuntunConnectionsDialog * self;
	g_return_val_if_fail (TUNTUN_IS_TUNTUN (tuntun), NULL);
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
	__params_it->name = "tuntun";
	g_value_init (&__params_it->value, TUNTUN_TYPE_TUNTUN);
	g_value_set_object (&__params_it->value, tuntun);
	__params_it++;
	self = g_object_newv (TUNTUN_TYPE_CONNECTIONS_DIALOG, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


void tuntun_connections_dialog_show (TuntunConnectionsDialog* self) {
	g_return_if_fail (TUNTUN_IS_CONNECTIONS_DIALOG (self));
	gtk_widget_show_all (GTK_WIDGET (self->priv->_window));
}


static void tuntun_connections_dialog_on_connection_added (TuntunConnectionsDialog* self, TuntunConnections* connections, TuntunConnection* connection) {
	g_return_if_fail (TUNTUN_IS_CONNECTIONS_DIALOG (self));
	g_return_if_fail (TUNTUN_IS_CONNECTIONS (connections));
	g_return_if_fail (TUNTUN_IS_CONNECTION (connection));
	tuntun_connections_dialog_store_add_item (self, connection);
}


static void tuntun_connections_dialog_on_connection_removed (TuntunConnectionsDialog* self, TuntunConnections* connections, TuntunConnection* connection) {
	g_return_if_fail (TUNTUN_IS_CONNECTIONS_DIALOG (self));
	g_return_if_fail (TUNTUN_IS_CONNECTIONS (connections));
	g_return_if_fail (TUNTUN_IS_CONNECTION (connection));
	tuntun_connections_dialog_store_remove_item (self, connection);
}


static void tuntun_connections_dialog_on_treeview_selection_changed (TuntunConnectionsDialog* self, GtkTreeSelection* sender) {
	gboolean sensitive;
	g_return_if_fail (TUNTUN_IS_CONNECTIONS_DIALOG (self));
	g_return_if_fail (GTK_IS_TREE_SELECTION (sender));
	sensitive = (tuntun_connections_dialog_get_selected_connection (self) != NULL);
	gtk_widget_set_sensitive (GTK_WIDGET (self->priv->button_conn_modify), sensitive);
	gtk_widget_set_sensitive (GTK_WIDGET (self->priv->button_conn_remove), sensitive);
}


static gboolean tuntun_connections_dialog_on_window_delete (TuntunConnectionsDialog* self, GtkWindow* sender, GdkEvent* event) {
	g_return_val_if_fail (TUNTUN_IS_CONNECTIONS_DIALOG (self), FALSE);
	g_return_val_if_fail (GTK_IS_WINDOW (sender), FALSE);
	tuntun_connections_dialog_cleanup_and_close (self);
	return TRUE;
}


static void tuntun_connections_dialog_on_button_connection_add_clicked (TuntunConnectionsDialog* self, GtkButton* sender) {
	TuntunConnectionInfo* connection_info;
	TuntunConnectionDialog* dialog;
	g_return_if_fail (TUNTUN_IS_CONNECTIONS_DIALOG (self));
	g_return_if_fail (GTK_IS_BUTTON (sender));
	connection_info = tuntun_connection_info_new ();
	dialog = tuntun_connection_dialog_new (connection_info);
	if (tuntun_connection_dialog_show (dialog, GTK_WIDGET (self->priv->_window)) == GTK_RESPONSE_OK) {
		TuntunConnection* connection;
		connection = tuntun_connection_new (connection_info);
		tuntun_connections_add (tuntun_tuntun_get_connections (self->priv->_tuntun), connection);
		self->priv->is_dirty = TRUE;
		(connection == NULL ? NULL : (connection = (g_object_unref (connection), NULL)));
	}
	(connection_info == NULL ? NULL : (connection_info = (g_object_unref (connection_info), NULL)));
	(dialog == NULL ? NULL : (dialog = (g_object_unref (dialog), NULL)));
}


static void tuntun_connections_dialog_on_button_connection_remove_clicked (TuntunConnectionsDialog* self, GtkButton* sender) {
	TuntunConnection* _tmp0;
	TuntunConnection* connection;
	GtkBuilder* _tmp1;
	GtkBuilder* builder;
	GtkDialog* _tmp2;
	GtkDialog* confirm;
	g_return_if_fail (TUNTUN_IS_CONNECTIONS_DIALOG (self));
	g_return_if_fail (GTK_IS_BUTTON (sender));
	_tmp0 = NULL;
	connection = (_tmp0 = tuntun_connections_dialog_get_selected_connection (self), (_tmp0 == NULL ? NULL : g_object_ref (_tmp0)));
	g_return_if_fail (connection != NULL);
	_tmp1 = NULL;
	builder = (_tmp1 = tuntun_utils_get_ui (), (_tmp1 == NULL ? NULL : g_object_ref (_tmp1)));
	_tmp2 = NULL;
	confirm = (_tmp2 = GTK_DIALOG (gtk_builder_get_object (builder, "dialog_connection_remove")), (_tmp2 == NULL ? NULL : g_object_ref (_tmp2)));
	if (gtk_dialog_run (confirm) == GTK_RESPONSE_OK) {
		tuntun_connections_remove (tuntun_tuntun_get_connections (self->priv->_tuntun), connection);
		self->priv->is_dirty = TRUE;
	}
	gtk_widget_hide (GTK_WIDGET (confirm));
	(connection == NULL ? NULL : (connection = (g_object_unref (connection), NULL)));
	(builder == NULL ? NULL : (builder = (g_object_unref (builder), NULL)));
	(confirm == NULL ? NULL : (confirm = (g_object_unref (confirm), NULL)));
}


static void tuntun_connections_dialog_on_button_connection_modify_clicked (TuntunConnectionsDialog* self, GtkButton* sender) {
	TuntunConnection* _tmp0;
	TuntunConnection* connection;
	TuntunConnectionDialog* dialog;
	g_return_if_fail (TUNTUN_IS_CONNECTIONS_DIALOG (self));
	g_return_if_fail (GTK_IS_BUTTON (sender));
	_tmp0 = NULL;
	connection = (_tmp0 = tuntun_connections_dialog_get_selected_connection (self), (_tmp0 == NULL ? NULL : g_object_ref (_tmp0)));
	if (connection == NULL) {
		(connection == NULL ? NULL : (connection = (g_object_unref (connection), NULL)));
		return;
	}
	dialog = tuntun_connection_dialog_new (tuntun_connection_get_info (connection));
	if (tuntun_connection_dialog_show (dialog, GTK_WIDGET (self->priv->_window)) == GTK_RESPONSE_OK) {
		GtkTreeIter iter = {0};
		if (tuntun_connections_dialog_store_find (self, connection, &iter)) {
			tuntun_connections_dialog_store_modify_item (self, &iter, connection);
		}
		self->priv->is_dirty = TRUE;
	}
	(connection == NULL ? NULL : (connection = (g_object_unref (connection), NULL)));
	(dialog == NULL ? NULL : (dialog = (g_object_unref (dialog), NULL)));
}


static void tuntun_connections_dialog_on_button_close_clicked (TuntunConnectionsDialog* self, GtkButton* sender) {
	GError * inner_error;
	g_return_if_fail (TUNTUN_IS_CONNECTIONS_DIALOG (self));
	g_return_if_fail (GTK_IS_BUTTON (sender));
	inner_error = NULL;
	if (self->priv->is_dirty) {
		{
			tuntun_connections_save (tuntun_tuntun_get_connections (self->priv->_tuntun), &inner_error);
			if (inner_error != NULL) {
				goto __catch8_g_error;
			}
		}
		goto __finally8;
		__catch8_g_error:
		{
			GError * err;
			err = inner_error;
			inner_error = NULL;
			{
				tuntun_utils_display_error (_ ("Error while saving config"), err->message);
				(err == NULL ? NULL : (err = (g_error_free (err), NULL)));
			}
		}
		__finally8:
		;
	}
	tuntun_connections_dialog_cleanup_and_close (self);
}


static void tuntun_connections_dialog_store_add_item (TuntunConnectionsDialog* self, TuntunConnection* connection) {
	GtkTreeIter iter = {0};
	g_return_if_fail (TUNTUN_IS_CONNECTIONS_DIALOG (self));
	g_return_if_fail (TUNTUN_IS_CONNECTION (connection));
	gtk_list_store_append (self->priv->_store, &iter);
	tuntun_connections_dialog_store_modify_item (self, &iter, connection);
}


static void tuntun_connections_dialog_store_modify_item (TuntunConnectionsDialog* self, GtkTreeIter* iter, TuntunConnection* connection) {
	GdkPixbuf* pixbuf;
	TuntunConnectionStates _tmp6;
	char* assigned_ip;
	g_return_if_fail (TUNTUN_IS_CONNECTIONS_DIALOG (self));
	g_return_if_fail (TUNTUN_IS_CONNECTION (connection));
	pixbuf = NULL;
	_tmp6 = tuntun_connection_get_status (connection);
	if (_tmp6 == TUNTUN_CONNECTION_STATES_DISCONNECTED)
	do {
		GdkPixbuf* _tmp1;
		GdkPixbuf* _tmp0;
		_tmp1 = NULL;
		_tmp0 = NULL;
		pixbuf = (_tmp1 = (_tmp0 = self->priv->_pixbufs[1], (_tmp0 == NULL ? NULL : g_object_ref (_tmp0))), (pixbuf == NULL ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL))), _tmp1);
		break;
	} while (0); else if (_tmp6 == TUNTUN_CONNECTION_STATES_CONNECTED)
	do {
		GdkPixbuf* _tmp3;
		GdkPixbuf* _tmp2;
		_tmp3 = NULL;
		_tmp2 = NULL;
		pixbuf = (_tmp3 = (_tmp2 = self->priv->_pixbufs[2], (_tmp2 == NULL ? NULL : g_object_ref (_tmp2))), (pixbuf == NULL ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL))), _tmp3);
		break;
	} while (0); else
	do {
		GdkPixbuf* _tmp5;
		GdkPixbuf* _tmp4;
		_tmp5 = NULL;
		_tmp4 = NULL;
		pixbuf = (_tmp5 = (_tmp4 = self->priv->_pixbufs[2], (_tmp4 == NULL ? NULL : g_object_ref (_tmp4))), (pixbuf == NULL ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL))), _tmp5);
		break;
	} while (0);
	assigned_ip = g_strdup ("");
	if (_vala_strcmp0 (tuntun_connection_info_get_assigned_ip (tuntun_connection_get_info (connection)), _ ("none")) != 0) {
		char* _tmp8;
		const char* _tmp7;
		_tmp8 = NULL;
		_tmp7 = NULL;
		assigned_ip = (_tmp8 = (_tmp7 = tuntun_connection_info_get_assigned_ip (tuntun_connection_get_info (connection)), (_tmp7 == NULL ? NULL : g_strdup (_tmp7))), (assigned_ip = (g_free (assigned_ip), NULL)), _tmp8);
	}
	gtk_list_store_set (self->priv->_store, &(*iter), TUNTUN_COLUMNS_NAME, tuntun_connection_info_get_name (tuntun_connection_get_info (connection)), TUNTUN_COLUMNS_ADDRESS, tuntun_connection_info_get_address (tuntun_connection_get_info (connection)), TUNTUN_COLUMNS_PORT, tuntun_connection_info_get_port (tuntun_connection_get_info (connection)), TUNTUN_COLUMNS_DOUBLE_CLICK_CONNECT, tuntun_connection_info_get_quick_connect (tuntun_connection_get_info (connection)), TUNTUN_COLUMNS_STATUS, pixbuf, TUNTUN_COLUMNS_ASSIGNED_IP, assigned_ip, TUNTUN_COLUMNS_CONNECTION, connection, -1);
	(pixbuf == NULL ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL)));
	assigned_ip = (g_free (assigned_ip), NULL);
}


static void tuntun_connections_dialog_store_remove_item (TuntunConnectionsDialog* self, TuntunConnection* connection) {
	GtkTreeIter iter = {0};
	g_return_if_fail (TUNTUN_IS_CONNECTIONS_DIALOG (self));
	g_return_if_fail (TUNTUN_IS_CONNECTION (connection));
	if (tuntun_connections_dialog_store_find (self, connection, &iter)) {
		gtk_list_store_remove (self->priv->_store, &iter);
	}
}


static gboolean tuntun_connections_dialog_store_find (TuntunConnectionsDialog* self, TuntunConnection* connection, GtkTreeIter* iter) {
	gboolean valid;
	gboolean result;
	g_return_val_if_fail (TUNTUN_IS_CONNECTIONS_DIALOG (self), FALSE);
	g_return_val_if_fail (TUNTUN_IS_CONNECTION (connection), FALSE);
	valid = FALSE;
	result = FALSE;
	/* Get the first iter in the list */
	valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (self->priv->_store), &(*iter));
	while (valid) {
		TuntunConnection* conn;
		conn = NULL;
		gtk_tree_model_get (GTK_TREE_MODEL (self->priv->_store), &(*iter), TUNTUN_COLUMNS_CONNECTION, &conn, -1, -1);
		if (conn == connection) {
			result = TRUE;
			break;
		}
		valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (self->priv->_store), &(*iter));
	}
	return result;
}


static void tuntun_connections_dialog_set_tuntun (TuntunConnectionsDialog* self, TuntunTuntun* value) {
	TuntunTuntun* _tmp1;
	TuntunTuntun* _tmp0;
	g_return_if_fail (TUNTUN_IS_CONNECTIONS_DIALOG (self));
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->_tuntun = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL ? NULL : g_object_ref (_tmp0))), (self->priv->_tuntun == NULL ? NULL : (self->priv->_tuntun = (g_object_unref (self->priv->_tuntun), NULL))), _tmp1);
	g_object_notify (((GObject *) (self)), "tuntun");
}


static TuntunConnection* tuntun_connections_dialog_get_selected_connection (TuntunConnectionsDialog* self) {
	GtkTreeSelection* _tmp0;
	GtkTreeSelection* selection;
	GtkTreeIter iter = {0};
	g_return_val_if_fail (TUNTUN_IS_CONNECTIONS_DIALOG (self), NULL);
	_tmp0 = NULL;
	selection = (_tmp0 = gtk_tree_view_get_selection (self->priv->_treeview), (_tmp0 == NULL ? NULL : g_object_ref (_tmp0)));
	if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
		TuntunConnection* connection;
		TuntunConnection* _tmp1;
		connection = NULL;
		gtk_tree_model_get (GTK_TREE_MODEL (self->priv->_store), &iter, TUNTUN_COLUMNS_CONNECTION, &connection, -1, -1);
		_tmp1 = NULL;
		return (_tmp1 = connection, (selection == NULL ? NULL : (selection = (g_object_unref (selection), NULL))), _tmp1);
	} else {
		TuntunConnection* _tmp2;
		_tmp2 = NULL;
		return (_tmp2 = NULL, (selection == NULL ? NULL : (selection = (g_object_unref (selection), NULL))), _tmp2);
	}
	(selection == NULL ? NULL : (selection = (g_object_unref (selection), NULL)));
}


static GObject * tuntun_connections_dialog_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	TuntunConnectionsDialogClass * klass;
	GObjectClass * parent_class;
	TuntunConnectionsDialog * self;
	klass = TUNTUN_CONNECTIONS_DIALOG_CLASS (g_type_class_peek (TUNTUN_TYPE_CONNECTIONS_DIALOG));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = TUNTUN_CONNECTIONS_DIALOG (obj);
	{
		tuntun_connections_dialog_initialize (self);
	}
	return obj;
}


static void tuntun_connections_dialog_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	TuntunConnectionsDialog * self;
	self = TUNTUN_CONNECTIONS_DIALOG (object);
	switch (property_id) {
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void tuntun_connections_dialog_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	TuntunConnectionsDialog * self;
	self = TUNTUN_CONNECTIONS_DIALOG (object);
	switch (property_id) {
		case TUNTUN_CONNECTIONS_DIALOG_TUNTUN:
		tuntun_connections_dialog_set_tuntun (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void tuntun_connections_dialog_class_init (TuntunConnectionsDialogClass * klass) {
	tuntun_connections_dialog_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (TuntunConnectionsDialogPrivate));
	G_OBJECT_CLASS (klass)->get_property = tuntun_connections_dialog_get_property;
	G_OBJECT_CLASS (klass)->set_property = tuntun_connections_dialog_set_property;
	G_OBJECT_CLASS (klass)->constructor = tuntun_connections_dialog_constructor;
	G_OBJECT_CLASS (klass)->dispose = tuntun_connections_dialog_dispose;
	g_object_class_install_property (G_OBJECT_CLASS (klass), TUNTUN_CONNECTIONS_DIALOG_TUNTUN, g_param_spec_object ("tuntun", "tuntun", "tuntun", TUNTUN_TYPE_TUNTUN, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_signal_new ("closed", TUNTUN_TYPE_CONNECTIONS_DIALOG, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}


static void tuntun_connections_dialog_instance_init (TuntunConnectionsDialog * self) {
	self->priv = TUNTUN_CONNECTIONS_DIALOG_GET_PRIVATE (self);
	self->priv->_window = NULL;
	self->priv->_treeview = NULL;
	self->priv->button_conn_remove = NULL;
	self->priv->button_conn_modify = NULL;
	self->priv->_tuntun = NULL;
	self->priv->is_dirty = FALSE;
}


static void tuntun_connections_dialog_dispose (GObject * obj) {
	TuntunConnectionsDialog * self;
	self = TUNTUN_CONNECTIONS_DIALOG (obj);
	(self->priv->_store == NULL ? NULL : (self->priv->_store = (g_object_unref (self->priv->_store), NULL)));
	(self->priv->_tuntun == NULL ? NULL : (self->priv->_tuntun = (g_object_unref (self->priv->_tuntun), NULL)));
	self->priv->_pixbufs = (_vala_array_free (self->priv->_pixbufs, self->priv->_pixbufs_length1, ((GDestroyNotify) (g_object_unref))), NULL);
	G_OBJECT_CLASS (tuntun_connections_dialog_parent_class)->dispose (obj);
}


GType tuntun_connections_dialog_get_type (void) {
	static volatile gsize tuntun_connections_dialog_type_id = 0;
	if (g_once_init_enter (&tuntun_connections_dialog_type_id)) {
		GType tuntun_connections_dialog_type_id_temp;
		static const GTypeInfo g_define_type_info = { sizeof (TuntunConnectionsDialogClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) tuntun_connections_dialog_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (TuntunConnectionsDialog), 0, (GInstanceInitFunc) tuntun_connections_dialog_instance_init };
		tuntun_connections_dialog_type_id_temp = g_type_register_static (G_TYPE_OBJECT, "TuntunConnectionsDialog", &g_define_type_info, 0);
		g_once_init_leave (&tuntun_connections_dialog_type_id, tuntun_connections_dialog_type_id_temp);
	}
	return tuntun_connections_dialog_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if (array != NULL && destroy_func != NULL) {
		int i;
		if (array_length >= 0)
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) (array))[i] != NULL)
			destroy_func (((gpointer*) (array))[i]);
		}
		else
		for (i = 0; ((gpointer*) (array))[i] != NULL; i = i + 1) {
			destroy_func (((gpointer*) (array))[i]);
		}
	}
	g_free (array);
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return (str1 != str2);
	}
	return strcmp (str1, str2);
}




