/*
 *  tuntun-connections.vala is a part of Tuntun
 *
 *  Tuntun: a simple applet to manage OpenVPN connections
 *
 *  Copyright (C) 2008  Andrea Del Signore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
 *
 *  Author:
 *     Andrea Del Signore <sejerpz@tin.it>       
 */

#ifndef __TUNTUN_CONNECTIONS_H__
#define __TUNTUN_CONNECTIONS_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include "tuntun-connection.h"

G_BEGIN_DECLS


#define TUNTUN_TYPE_CONNECTIONS (tuntun_connections_get_type ())
#define TUNTUN_CONNECTIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TUNTUN_TYPE_CONNECTIONS, TuntunConnections))
#define TUNTUN_CONNECTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TUNTUN_TYPE_CONNECTIONS, TuntunConnectionsClass))
#define TUNTUN_IS_CONNECTIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TUNTUN_TYPE_CONNECTIONS))
#define TUNTUN_IS_CONNECTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TUNTUN_TYPE_CONNECTIONS))
#define TUNTUN_CONNECTIONS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TUNTUN_TYPE_CONNECTIONS, TuntunConnectionsClass))

typedef struct _TuntunConnections TuntunConnections;
typedef struct _TuntunConnectionsClass TuntunConnectionsClass;
typedef struct _TuntunConnectionsPrivate TuntunConnectionsPrivate;

struct _TuntunConnections {
	GObject parent_instance;
	TuntunConnectionsPrivate * priv;
	GList* items;
};

struct _TuntunConnectionsClass {
	GObjectClass parent_class;
	void (*on_connection_activity) (TuntunConnections* self, TuntunConnection* connection);
	void (*on_connection_fatal_error) (TuntunConnections* self, TuntunConnection* connection, const char* error);
	void (*on_connection_authentication_required) (TuntunConnections* self, TuntunConnection* connection, TuntunAuthenticationModes mode, const char* type);
	void (*on_connection_authentication_failed) (TuntunConnections* self, TuntunConnection* connection, TuntunAuthenticationModes mode, const char* type);
	void (*on_connection_removed) (TuntunConnections* self, TuntunConnection* connection);
	void (*on_connection_added) (TuntunConnections* self, TuntunConnection* connection);
	void (*on_connection_status_changed) (TuntunConnections* self, TuntunConnection* connection);
};


void tuntun_connections_save (TuntunConnections* self, GError** error);
void tuntun_connections_add (TuntunConnections* self, TuntunConnection* connection);
void tuntun_connections_remove (TuntunConnections* self, TuntunConnection* connection);
void tuntun_connections_on_connection_activity (TuntunConnections* self, TuntunConnection* connection);
void tuntun_connections_on_connection_fatal_error (TuntunConnections* self, TuntunConnection* connection, const char* error);
void tuntun_connections_on_connection_authentication_required (TuntunConnections* self, TuntunConnection* connection, TuntunAuthenticationModes mode, const char* type);
void tuntun_connections_on_connection_authentication_failed (TuntunConnections* self, TuntunConnection* connection, TuntunAuthenticationModes mode, const char* type);
void tuntun_connections_on_connection_removed (TuntunConnections* self, TuntunConnection* connection);
void tuntun_connections_on_connection_added (TuntunConnections* self, TuntunConnection* connection);
void tuntun_connections_on_connection_status_changed (TuntunConnections* self, TuntunConnection* connection);
TuntunConnections* tuntun_connections_new (void);
GType tuntun_connections_get_type (void);


G_END_DECLS

#endif
