/*
 *  tuntun-log-window.vala is a part of Tuntun
 *
 *  Tuntun: a simple applet to manage OpenVPN connections
 *
 *  Copyright (C) 2008  Andrea Del Signore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
 *
 *  Author:
 *     Andrea Del Signore <sejerpz@tin.it>       
 */

#include "tuntun-log-window.h"
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <pango/pango.h>
#include <glib/gi18n-lib.h>
#include <float.h>
#include <math.h>
#include <gdk/gdk.h>
#include "tuntun-utils.h"
#include "tuntun-connections.h"
#include "tuntun-connection.h"
#include "tuntun-connection-info.h"


#define TUNTUN_TYPE_COMBO_COLUMNS (tuntun_combo_columns_get_type ())

typedef enum  {
	TUNTUN_COMBO_COLUMNS_NAME,
	TUNTUN_COMBO_COLUMNS_CONNECTION,
	TUNTUN_COMBO_COLUMNS_COUNT
} TuntunComboColumns;



GType tuntun_combo_columns_get_type (void);
struct _TuntunLogWindowPrivate {
	GtkWindow* _window;
	GtkTextView* _text_view;
	GtkEntry* _entry_send;
	GtkButton* _button_send;
	GtkComboBox* _combo;
	GtkListStore* _store;
	GtkTextTagTable* _tag_table;
	TuntunTuntun* _tuntun;
};

#define TUNTUN_LOG_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TUNTUN_TYPE_LOG_WINDOW, TuntunLogWindowPrivate))
enum  {
	TUNTUN_LOG_WINDOW_DUMMY_PROPERTY,
	TUNTUN_LOG_WINDOW_TUNTUN
};
static gboolean _tuntun_log_window_on_window_delete_gtk_widget_delete_event (GtkWindow* _sender, GdkEvent* event, gpointer self);
static void _tuntun_log_window_on_button_send_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self);
static void _tuntun_log_window_on_connection_data_received_tuntun_connection_control_channel_data_received (TuntunConnection* _sender, const char* data, gpointer self);
static void _tuntun_log_window_on_connection_data_sent_tuntun_connection_control_channel_data_sent (TuntunConnection* _sender, const char* data, gpointer self);
static void _tuntun_log_window_on_combo_active_connection_changed_gtk_combo_box_changed (GtkComboBox* _sender, gpointer self);
static void tuntun_log_window_initialize (TuntunLogWindow* self);
static void tuntun_log_window_cleanup_and_close (TuntunLogWindow* self);
static void tuntun_log_window_on_combo_active_connection_changed (TuntunLogWindow* self, GtkComboBox* sender);
static void tuntun_log_window_on_connection_data_sent (TuntunLogWindow* self, TuntunConnection* connection, const char* data);
static void tuntun_log_window_on_connection_data_received (TuntunLogWindow* self, TuntunConnection* connection, const char* data);
static void tuntun_log_window_append_text (TuntunLogWindow* self, GtkTextTag* tag, TuntunConnection* connection, const char* data);
static gboolean tuntun_log_window_on_window_delete (TuntunLogWindow* self, GtkWindow* sender, GdkEvent* event);
static void tuntun_log_window_on_button_send_clicked (TuntunLogWindow* self, GtkWidget* sender);
static void tuntun_log_window_send_command (TuntunLogWindow* self, TuntunConnection* connection, const char* command_text);
static void tuntun_log_window_set_tuntun (TuntunLogWindow* self, TuntunTuntun* value);
static TuntunConnection* tuntun_log_window_get_active_connection (TuntunLogWindow* self);
static GObject * tuntun_log_window_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer tuntun_log_window_parent_class = NULL;
static void tuntun_log_window_dispose (GObject * obj);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static int _vala_strcmp0 (const char * str1, const char * str2);




GType tuntun_combo_columns_get_type (void) {
	static GType tuntun_combo_columns_type_id = 0;
	if (G_UNLIKELY (tuntun_combo_columns_type_id == 0)) {
		static const GEnumValue values[] = {{TUNTUN_COMBO_COLUMNS_NAME, "TUNTUN_COMBO_COLUMNS_NAME", "name"}, {TUNTUN_COMBO_COLUMNS_CONNECTION, "TUNTUN_COMBO_COLUMNS_CONNECTION", "connection"}, {TUNTUN_COMBO_COLUMNS_COUNT, "TUNTUN_COMBO_COLUMNS_COUNT", "count"}, {0, NULL, NULL}};
		tuntun_combo_columns_type_id = g_enum_register_static ("TuntunComboColumns", values);
	}
	return tuntun_combo_columns_type_id;
}


TuntunLogWindow* tuntun_log_window_new (TuntunTuntun* tuntun) {
	GParameter * __params;
	GParameter * __params_it;
	TuntunLogWindow * self;
	g_return_val_if_fail (TUNTUN_IS_TUNTUN (tuntun), NULL);
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
	__params_it->name = "tuntun";
	g_value_init (&__params_it->value, TUNTUN_TYPE_TUNTUN);
	g_value_set_object (&__params_it->value, tuntun);
	__params_it++;
	self = g_object_newv (TUNTUN_TYPE_LOG_WINDOW, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


static gboolean _tuntun_log_window_on_window_delete_gtk_widget_delete_event (GtkWindow* _sender, GdkEvent* event, gpointer self) {
	return tuntun_log_window_on_window_delete (self, _sender, event);
}


static void _tuntun_log_window_on_button_send_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self) {
	tuntun_log_window_on_button_send_clicked (self, _sender);
}


static void _tuntun_log_window_on_connection_data_received_tuntun_connection_control_channel_data_received (TuntunConnection* _sender, const char* data, gpointer self) {
	tuntun_log_window_on_connection_data_received (self, _sender, data);
}


static void _tuntun_log_window_on_connection_data_sent_tuntun_connection_control_channel_data_sent (TuntunConnection* _sender, const char* data, gpointer self) {
	tuntun_log_window_on_connection_data_sent (self, _sender, data);
}


static void _tuntun_log_window_on_combo_active_connection_changed_gtk_combo_box_changed (GtkComboBox* _sender, gpointer self) {
	tuntun_log_window_on_combo_active_connection_changed (self, _sender);
}


static void tuntun_log_window_initialize (TuntunLogWindow* self) {
	GtkBuilder* _tmp0;
	GtkBuilder* builder;
	GtkTextTagTable* _tmp1;
	GtkTextTag* tag;
	GtkTextTag* _tmp2;
	GtkTextTag* _tmp3;
	GtkTextTag* _tmp4;
	GtkTextBuffer* text_buffer;
	GtkListStore* _tmp5;
	GtkTreeIter iter = {0};
	GtkCellRenderer* renderer;
	g_return_if_fail (TUNTUN_IS_LOG_WINDOW (self));
	_tmp0 = NULL;
	builder = (_tmp0 = tuntun_utils_get_ui (), (_tmp0 == NULL ? NULL : g_object_ref (_tmp0)));
	self->priv->_window = GTK_WINDOW (gtk_builder_get_object (builder, "window_log"));
	g_assert (self->priv->_window != NULL);
	/* one time initialization */
	g_signal_connect_object (GTK_WIDGET (self->priv->_window), "delete-event", ((GCallback) (_tuntun_log_window_on_window_delete_gtk_widget_delete_event)), self, 0);
	self->priv->_text_view = GTK_TEXT_VIEW (gtk_builder_get_object (builder, "textview_buffer"));
	g_assert (self->priv->_text_view != NULL);
	self->priv->_button_send = GTK_BUTTON (gtk_builder_get_object (builder, "button_send"));
	g_assert (self->priv->_button_send != NULL);
	g_signal_connect_object (self->priv->_button_send, "clicked", ((GCallback) (_tuntun_log_window_on_button_send_clicked_gtk_button_clicked)), self, 0);
	self->priv->_entry_send = GTK_ENTRY (gtk_builder_get_object (builder, "entry_send"));
	g_assert (self->priv->_entry_send != NULL);
	/* intialize text view formatting */
	_tmp1 = NULL;
	self->priv->_tag_table = (_tmp1 = gtk_text_tag_table_new (), (self->priv->_tag_table == NULL ? NULL : (self->priv->_tag_table = (g_object_unref (self->priv->_tag_table), NULL))), _tmp1);
	tag = gtk_text_tag_new ("information");
	g_object_set (G_OBJECT (tag), "foreground", "DimGray", "style", PANGO_STYLE_ITALIC, "weight", PANGO_WEIGHT_BOLD, "family", "Monospace", NULL);
	gtk_text_tag_table_add (self->priv->_tag_table, tag);
	_tmp2 = NULL;
	tag = (_tmp2 = gtk_text_tag_new ("received"), (tag == NULL ? NULL : (tag = (g_object_unref (tag), NULL))), _tmp2);
	g_object_set (G_OBJECT (tag), "style", PANGO_STYLE_NORMAL, "family", "Monospace", NULL);
	gtk_text_tag_table_add (self->priv->_tag_table, tag);
	_tmp3 = NULL;
	tag = (_tmp3 = gtk_text_tag_new ("sent"), (tag == NULL ? NULL : (tag = (g_object_unref (tag), NULL))), _tmp3);
	g_object_set (G_OBJECT (tag), "foreground", "DimGray", "style", PANGO_STYLE_NORMAL, "family", "Monospace", NULL);
	gtk_text_tag_table_add (self->priv->_tag_table, tag);
	_tmp4 = NULL;
	tag = (_tmp4 = gtk_text_tag_new ("connection-name"), (tag == NULL ? NULL : (tag = (g_object_unref (tag), NULL))), _tmp4);
	g_object_set (G_OBJECT (tag), "weight", PANGO_WEIGHT_BOLD, "family", "Monospace", NULL);
	gtk_text_tag_table_add (self->priv->_tag_table, tag);
	text_buffer = gtk_text_buffer_new (self->priv->_tag_table);
	gtk_text_view_set_buffer (self->priv->_text_view, text_buffer);
	/* initialize connection list view */
	_tmp5 = NULL;
	self->priv->_store = (_tmp5 = gtk_list_store_new (((gint) (TUNTUN_COMBO_COLUMNS_COUNT)), G_TYPE_STRING, TUNTUN_TYPE_CONNECTION, NULL), (self->priv->_store == NULL ? NULL : (self->priv->_store = (g_object_unref (self->priv->_store), NULL))), _tmp5);
	gtk_list_store_append (self->priv->_store, &iter);
	gtk_list_store_set (self->priv->_store, &iter, TUNTUN_COMBO_COLUMNS_NAME, _ ("All"), TUNTUN_COMBO_COLUMNS_CONNECTION, NULL, -1);
	{
		GList* connection_collection;
		GList* connection_it;
		connection_collection = tuntun_tuntun_get_connections (self->priv->_tuntun)->items;
		for (connection_it = connection_collection; connection_it != NULL; connection_it = connection_it->next) {
			TuntunConnection* _tmp6;
			TuntunConnection* connection;
			_tmp6 = NULL;
			connection = (_tmp6 = ((TuntunConnection*) (connection_it->data)), (_tmp6 == NULL ? NULL : g_object_ref (_tmp6)));
			{
				gtk_list_store_append (self->priv->_store, &iter);
				gtk_list_store_set (self->priv->_store, &iter, TUNTUN_COMBO_COLUMNS_NAME, tuntun_connection_info_get_name (tuntun_connection_get_info (connection)), TUNTUN_COMBO_COLUMNS_CONNECTION, connection, -1);
				g_signal_connect_object (connection, "control-channel-data-received", ((GCallback) (_tuntun_log_window_on_connection_data_received_tuntun_connection_control_channel_data_received)), self, 0);
				g_signal_connect_object (connection, "control-channel-data-sent", ((GCallback) (_tuntun_log_window_on_connection_data_sent_tuntun_connection_control_channel_data_sent)), self, 0);
				(connection == NULL ? NULL : (connection = (g_object_unref (connection), NULL)));
			}
		}
	}
	self->priv->_combo = GTK_COMBO_BOX (gtk_builder_get_object (builder, "combobox_connection"));
	g_assert (self->priv->_combo != NULL);
	renderer = GTK_CELL_RENDERER (g_object_ref_sink (((GtkCellRendererText*) (gtk_cell_renderer_text_new ()))));
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (self->priv->_combo), renderer, TRUE);
	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (self->priv->_combo), renderer, "text", 0, NULL);
	gtk_combo_box_set_model (self->priv->_combo, GTK_TREE_MODEL (self->priv->_store));
	g_signal_connect_object (self->priv->_combo, "changed", ((GCallback) (_tuntun_log_window_on_combo_active_connection_changed_gtk_combo_box_changed)), self, 0);
	gtk_tree_model_get_iter_first (GTK_TREE_MODEL (self->priv->_store), &iter);
	gtk_combo_box_set_active_iter (self->priv->_combo, &iter);
	(builder == NULL ? NULL : (builder = (g_object_unref (builder), NULL)));
	(tag == NULL ? NULL : (tag = (g_object_unref (tag), NULL)));
	(text_buffer == NULL ? NULL : (text_buffer = (g_object_unref (text_buffer), NULL)));
	(renderer == NULL ? NULL : (renderer = (g_object_unref (renderer), NULL)));
}


static void tuntun_log_window_cleanup_and_close (TuntunLogWindow* self) {
	guint _tmp0;
	guint _tmp1;
	GtkTreeIter iter = {0};
	gboolean valid;
	guint _tmp4;
	GtkListStore* _tmp5;
	GtkTextTagTable* _tmp6;
	TuntunTuntun* _tmp7;
	g_return_if_fail (TUNTUN_IS_LOG_WINDOW (self));
	g_signal_handlers_disconnect_matched (GTK_WIDGET (self->priv->_window), G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("delete-event", GTK_TYPE_WIDGET, &_tmp0, NULL, FALSE), _tmp0), 0, NULL, ((GCallback) (_tuntun_log_window_on_window_delete_gtk_widget_delete_event)), self);
	gtk_widget_hide (GTK_WIDGET (self->priv->_window));
	g_signal_handlers_disconnect_matched (self->priv->_button_send, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("clicked", GTK_TYPE_BUTTON, &_tmp1, NULL, FALSE), _tmp1), 0, NULL, ((GCallback) (_tuntun_log_window_on_button_send_clicked_gtk_button_clicked)), self);
	valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (self->priv->_store), &iter);
	while (valid) {
		TuntunConnection* connection;
		connection = NULL;
		gtk_tree_model_get (GTK_TREE_MODEL (self->priv->_store), &iter, TUNTUN_COMBO_COLUMNS_CONNECTION, &connection, -1);
		if (connection != NULL) {
			guint _tmp2;
			guint _tmp3;
			g_signal_handlers_disconnect_matched (connection, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("control-channel-data-received", TUNTUN_TYPE_CONNECTION, &_tmp2, NULL, FALSE), _tmp2), 0, NULL, ((GCallback) (_tuntun_log_window_on_connection_data_received_tuntun_connection_control_channel_data_received)), self);
			g_signal_handlers_disconnect_matched (connection, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("control-channel-data-sent", TUNTUN_TYPE_CONNECTION, &_tmp3, NULL, FALSE), _tmp3), 0, NULL, ((GCallback) (_tuntun_log_window_on_connection_data_sent_tuntun_connection_control_channel_data_sent)), self);
		}
		valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (self->priv->_store), &iter);
		(connection == NULL ? NULL : (connection = (g_object_unref (connection), NULL)));
	}
	gtk_combo_box_set_model (self->priv->_combo, NULL);
	g_signal_handlers_disconnect_matched (self->priv->_combo, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("changed", GTK_TYPE_COMBO_BOX, &_tmp4, NULL, FALSE), _tmp4), 0, NULL, ((GCallback) (_tuntun_log_window_on_combo_active_connection_changed_gtk_combo_box_changed)), self);
	gtk_cell_layout_clear (GTK_CELL_LAYOUT (self->priv->_combo));
	_tmp5 = NULL;
	self->priv->_store = (_tmp5 = NULL, (self->priv->_store == NULL ? NULL : (self->priv->_store = (g_object_unref (self->priv->_store), NULL))), _tmp5);
	_tmp6 = NULL;
	self->priv->_tag_table = (_tmp6 = NULL, (self->priv->_tag_table == NULL ? NULL : (self->priv->_tag_table = (g_object_unref (self->priv->_tag_table), NULL))), _tmp6);
	_tmp7 = NULL;
	self->priv->_tuntun = (_tmp7 = NULL, (self->priv->_tuntun == NULL ? NULL : (self->priv->_tuntun = (g_object_unref (self->priv->_tuntun), NULL))), _tmp7);
	g_signal_emit_by_name (G_OBJECT (self), "closed");
}


static void tuntun_log_window_on_combo_active_connection_changed (TuntunLogWindow* self, GtkComboBox* sender) {
	g_return_if_fail (TUNTUN_IS_LOG_WINDOW (self));
	g_return_if_fail (GTK_IS_COMBO_BOX (sender));
	if (tuntun_log_window_get_active_connection (self) == NULL) {
		gtk_widget_set_sensitive (GTK_WIDGET (self->priv->_entry_send), FALSE);
		gtk_widget_set_sensitive (GTK_WIDGET (self->priv->_button_send), FALSE);
		gtk_entry_set_text (self->priv->_entry_send, _ ("select a connection first..."));
	} else {
		gtk_widget_set_sensitive (GTK_WIDGET (self->priv->_entry_send), TRUE);
		gtk_widget_set_sensitive (GTK_WIDGET (self->priv->_button_send), TRUE);
		gtk_entry_set_text (self->priv->_entry_send, "");
	}
}


static void tuntun_log_window_on_connection_data_sent (TuntunLogWindow* self, TuntunConnection* connection, const char* data) {
	GtkTextTag* tag;
	GtkTextTag* _tmp1;
	GtkTextTag* _tmp0;
	g_return_if_fail (TUNTUN_IS_LOG_WINDOW (self));
	g_return_if_fail (TUNTUN_IS_CONNECTION (connection));
	g_return_if_fail (data != NULL);
	tag = NULL;
	_tmp1 = NULL;
	_tmp0 = NULL;
	tag = (_tmp1 = (_tmp0 = gtk_text_tag_table_lookup (self->priv->_tag_table, "sent"), (_tmp0 == NULL ? NULL : g_object_ref (_tmp0))), (tag == NULL ? NULL : (tag = (g_object_unref (tag), NULL))), _tmp1);
	tuntun_log_window_append_text (self, tag, connection, data);
	(tag == NULL ? NULL : (tag = (g_object_unref (tag), NULL)));
}


static void tuntun_log_window_on_connection_data_received (TuntunLogWindow* self, TuntunConnection* connection, const char* data) {
	GtkTextTag* tag;
	GtkTextTag* _tmp1;
	GtkTextTag* _tmp0;
	g_return_if_fail (TUNTUN_IS_LOG_WINDOW (self));
	g_return_if_fail (TUNTUN_IS_CONNECTION (connection));
	g_return_if_fail (data != NULL);
	tag = NULL;
	_tmp1 = NULL;
	_tmp0 = NULL;
	tag = (_tmp1 = (_tmp0 = gtk_text_tag_table_lookup (self->priv->_tag_table, "received"), (_tmp0 == NULL ? NULL : g_object_ref (_tmp0))), (tag == NULL ? NULL : (tag = (g_object_unref (tag), NULL))), _tmp1);
	tuntun_log_window_append_text (self, tag, connection, data);
	(tag == NULL ? NULL : (tag = (g_object_unref (tag), NULL)));
}


static void tuntun_log_window_append_text (TuntunLogWindow* self, GtkTextTag* tag, TuntunConnection* connection, const char* data) {
	GtkTextIter iter = {0};
	GtkTextIter end_iter = {0};
	GtkTextBuffer* _tmp0;
	GtkTextBuffer* text_buffer;
	GtkTextTag* tag_connection;
	GtkTextTag* _tmp2;
	GtkTextTag* _tmp1;
	char** _tmp3;
	gint lines_length1;
	char** lines;
	char* tmp;
	gint line_count;
	GtkTextMark* _tmp8;
	GtkTextMark* mark;
	g_return_if_fail (TUNTUN_IS_LOG_WINDOW (self));
	g_return_if_fail (GTK_IS_TEXT_TAG (tag));
	g_return_if_fail (TUNTUN_IS_CONNECTION (connection));
	g_return_if_fail (data != NULL);
	_tmp0 = NULL;
	text_buffer = (_tmp0 = gtk_text_view_get_buffer (self->priv->_text_view), (_tmp0 == NULL ? NULL : g_object_ref (_tmp0)));
	g_return_if_fail (text_buffer != NULL);
	tag_connection = NULL;
	_tmp2 = NULL;
	_tmp1 = NULL;
	tag_connection = (_tmp2 = (_tmp1 = gtk_text_tag_table_lookup (self->priv->_tag_table, "connection-name"), (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (tag_connection == NULL ? NULL : (tag_connection = (g_object_unref (tag_connection), NULL))), _tmp2);
	gtk_text_buffer_get_end_iter (text_buffer, &iter);
	_tmp3 = NULL;
	lines = (_tmp3 = g_strsplit (data, "\n", 0), lines_length1 = -1, _tmp3);
	tmp = NULL;
	line_count = gtk_text_buffer_get_line_count (text_buffer);
	{
		char** line_collection;
		int line_collection_length1;
		int line_it;
		line_collection = lines;
		line_collection_length1 = lines_length1;
		for (line_it = 0; (lines_length1 != -1 && line_it < lines_length1) || (lines_length1 == -1 && line_collection[line_it] != NULL); line_it = line_it + 1) {
			const char* _tmp7;
			char* line;
			_tmp7 = NULL;
			line = (_tmp7 = line_collection[line_it], (_tmp7 == NULL ? NULL : g_strdup (_tmp7)));
			{
				char* _tmp5;
				char* _tmp6;
				if (line == NULL) {
					line = (g_free (line), NULL);
					break;
				}
				if (_vala_strcmp0 (line, "") == 0) {
					line = (g_free (line), NULL);
					continue;
				}
				if (!g_str_has_suffix (line, "\n")) {
					char* _tmp4;
					_tmp4 = NULL;
					line = (_tmp4 = g_strconcat (line, ("\n"), NULL), (line = (g_free (line), NULL)), _tmp4);
				}
				if (line_count > 500) {
					gtk_text_buffer_get_start_iter (text_buffer, &iter);
					gtk_text_buffer_get_iter_at_line (text_buffer, &end_iter, 1);
					gtk_text_buffer_delete (text_buffer, &iter, &end_iter);
					gtk_text_buffer_get_end_iter (text_buffer, &iter);
					line_count--;
				}
				_tmp5 = NULL;
				tmp = (_tmp5 = g_strdup_printf ("%s: ", tuntun_connection_info_get_name (tuntun_connection_get_info (connection))), (tmp = (g_free (tmp), NULL)), _tmp5);
				gtk_text_buffer_insert_with_tags (text_buffer, &iter, tmp, -1, tag_connection, NULL);
				_tmp6 = NULL;
				tmp = (_tmp6 = g_strdup_printf ("%s", line), (tmp = (g_free (tmp), NULL)), _tmp6);
				gtk_text_buffer_insert_with_tags (text_buffer, &iter, tmp, -1, tag, NULL);
				line_count++;
				line = (g_free (line), NULL);
			}
		}
	}
	_tmp8 = NULL;
	mark = (_tmp8 = gtk_text_buffer_create_mark (text_buffer, NULL, &iter, FALSE), (_tmp8 == NULL ? NULL : g_object_ref (_tmp8)));
	gtk_text_view_scroll_to_iter (self->priv->_text_view, &iter, ((double) (0)), FALSE, ((double) (0)), ((double) (0)));
	gtk_text_view_scroll_to_mark (self->priv->_text_view, mark, ((double) (0)), FALSE, ((double) (0)), ((double) (0)));
	gtk_text_buffer_delete_mark (text_buffer, mark);
	(text_buffer == NULL ? NULL : (text_buffer = (g_object_unref (text_buffer), NULL)));
	(tag_connection == NULL ? NULL : (tag_connection = (g_object_unref (tag_connection), NULL)));
	lines = (_vala_array_free (lines, lines_length1, ((GDestroyNotify) (g_free))), NULL);
	tmp = (g_free (tmp), NULL);
	(mark == NULL ? NULL : (mark = (g_object_unref (mark), NULL)));
}


static gboolean tuntun_log_window_on_window_delete (TuntunLogWindow* self, GtkWindow* sender, GdkEvent* event) {
	g_return_val_if_fail (TUNTUN_IS_LOG_WINDOW (self), FALSE);
	g_return_val_if_fail (GTK_IS_WINDOW (sender), FALSE);
	tuntun_log_window_cleanup_and_close (self);
	return TRUE;
}


static void tuntun_log_window_on_button_send_clicked (TuntunLogWindow* self, GtkWidget* sender) {
	const char* _tmp0;
	char* command_text;
	TuntunConnection* _tmp1;
	TuntunConnection* connection;
	g_return_if_fail (TUNTUN_IS_LOG_WINDOW (self));
	g_return_if_fail (GTK_IS_WIDGET (sender));
	_tmp0 = NULL;
	command_text = (_tmp0 = gtk_entry_get_text (self->priv->_entry_send), (_tmp0 == NULL ? NULL : g_strdup (_tmp0)));
	_tmp1 = NULL;
	connection = (_tmp1 = tuntun_log_window_get_active_connection (self), (_tmp1 == NULL ? NULL : g_object_ref (_tmp1)));
	tuntun_log_window_send_command (self, tuntun_log_window_get_active_connection (self), command_text);
	gtk_entry_set_text (self->priv->_entry_send, "");
	command_text = (g_free (command_text), NULL);
	(connection == NULL ? NULL : (connection = (g_object_unref (connection), NULL)));
}


static void tuntun_log_window_send_command (TuntunLogWindow* self, TuntunConnection* connection, const char* command_text) {
	char* _tmp0;
	g_return_if_fail (TUNTUN_IS_LOG_WINDOW (self));
	g_return_if_fail (TUNTUN_IS_CONNECTION (connection));
	g_return_if_fail (command_text != NULL);
	g_return_if_fail (connection != NULL);
	g_return_if_fail (command_text != NULL);
	_tmp0 = NULL;
	tuntun_connection_send (connection, (_tmp0 = g_strdup_printf ("%s\n", command_text)));
	_tmp0 = (g_free (_tmp0), NULL);
}


void tuntun_log_window_show (TuntunLogWindow* self) {
	g_return_if_fail (TUNTUN_IS_LOG_WINDOW (self));
	gtk_widget_show_all (GTK_WIDGET (self->priv->_window));
}


static void tuntun_log_window_set_tuntun (TuntunLogWindow* self, TuntunTuntun* value) {
	TuntunTuntun* _tmp1;
	TuntunTuntun* _tmp0;
	g_return_if_fail (TUNTUN_IS_LOG_WINDOW (self));
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->_tuntun = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL ? NULL : g_object_ref (_tmp0))), (self->priv->_tuntun == NULL ? NULL : (self->priv->_tuntun = (g_object_unref (self->priv->_tuntun), NULL))), _tmp1);
	g_object_notify (((GObject *) (self)), "tuntun");
}


static TuntunConnection* tuntun_log_window_get_active_connection (TuntunLogWindow* self) {
	GtkTreeIter iter = {0};
	TuntunConnection* connection;
	g_return_val_if_fail (TUNTUN_IS_LOG_WINDOW (self), NULL);
	connection = NULL;
	if (gtk_combo_box_get_active_iter (self->priv->_combo, &iter)) {
		GtkTreeModel* _tmp0;
		GtkTreeModel* store;
		_tmp0 = NULL;
		store = (_tmp0 = gtk_combo_box_get_model (self->priv->_combo), (_tmp0 == NULL ? NULL : g_object_ref (_tmp0)));
		gtk_tree_model_get (store, &iter, TUNTUN_COMBO_COLUMNS_CONNECTION, &connection, -1);
		(store == NULL ? NULL : (store = (g_object_unref (store), NULL)));
	}
	return connection;
}


static GObject * tuntun_log_window_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	TuntunLogWindowClass * klass;
	GObjectClass * parent_class;
	TuntunLogWindow * self;
	klass = TUNTUN_LOG_WINDOW_CLASS (g_type_class_peek (TUNTUN_TYPE_LOG_WINDOW));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = TUNTUN_LOG_WINDOW (obj);
	{
		tuntun_log_window_initialize (self);
	}
	return obj;
}


static void tuntun_log_window_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	TuntunLogWindow * self;
	self = TUNTUN_LOG_WINDOW (object);
	switch (property_id) {
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void tuntun_log_window_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	TuntunLogWindow * self;
	self = TUNTUN_LOG_WINDOW (object);
	switch (property_id) {
		case TUNTUN_LOG_WINDOW_TUNTUN:
		tuntun_log_window_set_tuntun (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void tuntun_log_window_class_init (TuntunLogWindowClass * klass) {
	tuntun_log_window_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (TuntunLogWindowPrivate));
	G_OBJECT_CLASS (klass)->get_property = tuntun_log_window_get_property;
	G_OBJECT_CLASS (klass)->set_property = tuntun_log_window_set_property;
	G_OBJECT_CLASS (klass)->constructor = tuntun_log_window_constructor;
	G_OBJECT_CLASS (klass)->dispose = tuntun_log_window_dispose;
	g_object_class_install_property (G_OBJECT_CLASS (klass), TUNTUN_LOG_WINDOW_TUNTUN, g_param_spec_object ("tuntun", "tuntun", "tuntun", TUNTUN_TYPE_TUNTUN, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_signal_new ("closed", TUNTUN_TYPE_LOG_WINDOW, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}


static void tuntun_log_window_instance_init (TuntunLogWindow * self) {
	self->priv = TUNTUN_LOG_WINDOW_GET_PRIVATE (self);
	self->priv->_window = NULL;
	self->priv->_text_view = NULL;
	self->priv->_entry_send = NULL;
	self->priv->_button_send = NULL;
	self->priv->_combo = NULL;
	self->priv->_store = NULL;
	self->priv->_tag_table = NULL;
}


static void tuntun_log_window_dispose (GObject * obj) {
	TuntunLogWindow * self;
	self = TUNTUN_LOG_WINDOW (obj);
	(self->priv->_store == NULL ? NULL : (self->priv->_store = (g_object_unref (self->priv->_store), NULL)));
	(self->priv->_tag_table == NULL ? NULL : (self->priv->_tag_table = (g_object_unref (self->priv->_tag_table), NULL)));
	(self->priv->_tuntun == NULL ? NULL : (self->priv->_tuntun = (g_object_unref (self->priv->_tuntun), NULL)));
	G_OBJECT_CLASS (tuntun_log_window_parent_class)->dispose (obj);
}


GType tuntun_log_window_get_type (void) {
	static volatile gsize tuntun_log_window_type_id = 0;
	if (g_once_init_enter (&tuntun_log_window_type_id)) {
		GType tuntun_log_window_type_id_temp;
		static const GTypeInfo g_define_type_info = { sizeof (TuntunLogWindowClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) tuntun_log_window_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (TuntunLogWindow), 0, (GInstanceInitFunc) tuntun_log_window_instance_init };
		tuntun_log_window_type_id_temp = g_type_register_static (G_TYPE_OBJECT, "TuntunLogWindow", &g_define_type_info, 0);
		g_once_init_leave (&tuntun_log_window_type_id, tuntun_log_window_type_id_temp);
	}
	return tuntun_log_window_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if (array != NULL && destroy_func != NULL) {
		int i;
		if (array_length >= 0)
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) (array))[i] != NULL)
			destroy_func (((gpointer*) (array))[i]);
		}
		else
		for (i = 0; ((gpointer*) (array))[i] != NULL; i = i + 1) {
			destroy_func (((gpointer*) (array))[i]);
		}
	}
	g_free (array);
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return (str1 != str2);
	}
	return strcmp (str1, str2);
}




