
#include "tuntun-tooltip.h"
#include <stdlib.h>
#include <string.h>
#include <glib/gi18n-lib.h>
#include "tuntun-connection.h"
#include "tuntun-utils.h"
#include "tuntun-connection-info.h"
#include "tuntun-constants.h"




struct _TuntunTooltipPrivate {
	TuntunConnections* _connections;
};

#define TUNTUN_TOOLTIP_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TUNTUN_TYPE_TOOLTIP, TuntunTooltipPrivate))
enum  {
	TUNTUN_TOOLTIP_DUMMY_PROPERTY,
	TUNTUN_TOOLTIP_CONNECTIONS
};
static void tuntun_tooltip_add_title (TuntunTooltip* self);
static void tuntun_tooltip_add_connection (TuntunTooltip* self, TuntunConnection* connection, const char* status_image);
static TuntunConnections* tuntun_tooltip_get_connections (TuntunTooltip* self);
static void tuntun_tooltip_set_connections (TuntunTooltip* self, TuntunConnections* value);
static GObject * tuntun_tooltip_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer tuntun_tooltip_parent_class = NULL;
static void tuntun_tooltip_dispose (GObject * obj);



static void tuntun_tooltip_add_title (TuntunTooltip* self) {
	GtkLabel* title;
	char* _tmp0;
	g_return_if_fail (TUNTUN_IS_TOOLTIP (self));
	title = g_object_ref_sink (((GtkLabel*) (gtk_label_new (NULL))));
	_tmp0 = NULL;
	gtk_label_set_markup (title, (_tmp0 = g_strdup_printf ("<b>%s</b>", _ ("Tuntun connections status"))));
	_tmp0 = (g_free (_tmp0), NULL);
	gtk_box_pack_start (GTK_BOX (self), GTK_WIDGET (title), FALSE, TRUE, ((guint) (8)));
	(title == NULL ? NULL : (title = (g_object_unref (title), NULL)));
}


static void tuntun_tooltip_add_connection (TuntunTooltip* self, TuntunConnection* connection, const char* status_image) {
	char* _tmp0;
	GtkImage* _tmp1;
	GtkImage* widget;
	GtkHBox* hbox;
	GtkLabel* label;
	GtkLabel* _tmp2;
	g_return_if_fail (TUNTUN_IS_TOOLTIP (self));
	g_return_if_fail (TUNTUN_IS_CONNECTION (connection));
	g_return_if_fail (status_image != NULL);
	_tmp0 = NULL;
	_tmp1 = NULL;
	widget = (_tmp1 = g_object_ref_sink (((GtkImage*) (gtk_image_new_from_file ((_tmp0 = tuntun_utils_get_image_path (status_image)))))), (_tmp0 = (g_free (_tmp0), NULL)), _tmp1);
	hbox = g_object_ref_sink (((GtkHBox*) (gtk_hbox_new (FALSE, 8))));
	label = NULL;
	gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET (widget), FALSE, FALSE, ((guint) (4)));
	_tmp2 = NULL;
	gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET ((_tmp2 = g_object_ref_sink (((GtkLabel*) (gtk_label_new_with_mnemonic (tuntun_connection_info_get_name (tuntun_connection_get_info (connection)))))))), FALSE, FALSE, ((guint) (12)));
	(_tmp2 == NULL ? NULL : (_tmp2 = (g_object_unref (_tmp2), NULL)));
	if (tuntun_connection_get_status (connection) == TUNTUN_CONNECTION_STATES_CONNECTED) {
		GtkLabel* _tmp3;
		char* _tmp5;
		char* _tmp4;
		_tmp3 = NULL;
		label = (_tmp3 = g_object_ref_sink (((GtkLabel*) (gtk_label_new (NULL)))), (label == NULL ? NULL : (label = (g_object_unref (label), NULL))), _tmp3);
		_tmp5 = NULL;
		_tmp4 = NULL;
		gtk_label_set_markup (label, (_tmp5 = g_strdup_printf ("<i>%s</i>", (_tmp4 = g_strdup_printf (_ ("ip address: %s"), tuntun_connection_info_get_assigned_ip (tuntun_connection_get_info (connection)))))));
		_tmp5 = (g_free (_tmp5), NULL);
		_tmp4 = (g_free (_tmp4), NULL);
		gtk_box_pack_end (GTK_BOX (hbox), GTK_WIDGET (label), FALSE, FALSE, ((guint) (12)));
	} else {
		GtkLabel* _tmp6;
		char* _tmp7;
		_tmp6 = NULL;
		label = (_tmp6 = g_object_ref_sink (((GtkLabel*) (gtk_label_new (NULL)))), (label == NULL ? NULL : (label = (g_object_unref (label), NULL))), _tmp6);
		_tmp7 = NULL;
		gtk_label_set_markup (label, (_tmp7 = g_strdup_printf ("<i>%s</i>", _ ("disconnected"))));
		_tmp7 = (g_free (_tmp7), NULL);
		gtk_box_pack_end (GTK_BOX (hbox), GTK_WIDGET (label), FALSE, FALSE, ((guint) (12)));
	}
	gtk_box_pack_start (GTK_BOX (self), GTK_WIDGET (hbox), TRUE, TRUE, ((guint) (4)));
	(widget == NULL ? NULL : (widget = (g_object_unref (widget), NULL)));
	(hbox == NULL ? NULL : (hbox = (g_object_unref (hbox), NULL)));
	(label == NULL ? NULL : (label = (g_object_unref (label), NULL)));
}


TuntunTooltip* tuntun_tooltip_new (TuntunConnections* connections) {
	GParameter * __params;
	GParameter * __params_it;
	TuntunTooltip * self;
	g_return_val_if_fail (TUNTUN_IS_CONNECTIONS (connections), NULL);
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
	__params_it->name = "connections";
	g_value_init (&__params_it->value, TUNTUN_TYPE_CONNECTIONS);
	g_value_set_object (&__params_it->value, connections);
	__params_it++;
	self = g_object_newv (TUNTUN_TYPE_TOOLTIP, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


static TuntunConnections* tuntun_tooltip_get_connections (TuntunTooltip* self) {
	g_return_val_if_fail (TUNTUN_IS_TOOLTIP (self), NULL);
	return self->priv->_connections;
}


static void tuntun_tooltip_set_connections (TuntunTooltip* self, TuntunConnections* value) {
	TuntunConnections* _tmp2;
	TuntunConnections* _tmp1;
	g_return_if_fail (TUNTUN_IS_TOOLTIP (self));
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_connections = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->_connections == NULL ? NULL : (self->priv->_connections = (g_object_unref (self->priv->_connections), NULL))), _tmp2);
	g_object_notify (((GObject *) (self)), "connections");
}


static GObject * tuntun_tooltip_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	TuntunTooltipClass * klass;
	GObjectClass * parent_class;
	TuntunTooltip * self;
	klass = TUNTUN_TOOLTIP_CLASS (g_type_class_peek (TUNTUN_TYPE_TOOLTIP));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = TUNTUN_TOOLTIP (obj);
	{
		gboolean title_added;
		title_added = FALSE;
		{
			GList* connection_collection;
			GList* connection_it;
			connection_collection = self->priv->_connections->items;
			for (connection_it = connection_collection; connection_it != NULL; connection_it = connection_it->next) {
				TuntunConnection* _tmp0;
				TuntunConnection* connection;
				_tmp0 = NULL;
				connection = (_tmp0 = ((TuntunConnection*) (connection_it->data)), (_tmp0 == NULL ? NULL : g_object_ref (_tmp0)));
				{
					if (tuntun_connection_get_status (connection) == TUNTUN_CONNECTION_STATES_CONNECTED) {
						if (!title_added) {
							tuntun_tooltip_add_title (self);
							title_added = TRUE;
						}
						tuntun_tooltip_add_connection (self, connection, TUNTUN_CONSTANTS_IMAGES_CONNECTION_STATUS_CONNECT);
					}
					(connection == NULL ? NULL : (connection = (g_object_unref (connection), NULL)));
				}
			}
		}
		{
			GList* connection_collection;
			GList* connection_it;
			connection_collection = self->priv->_connections->items;
			for (connection_it = connection_collection; connection_it != NULL; connection_it = connection_it->next) {
				TuntunConnection* _tmp1;
				TuntunConnection* connection;
				_tmp1 = NULL;
				connection = (_tmp1 = ((TuntunConnection*) (connection_it->data)), (_tmp1 == NULL ? NULL : g_object_ref (_tmp1)));
				{
					if (tuntun_connection_get_status (connection) == TUNTUN_CONNECTION_STATES_DISCONNECTED) {
						if (!title_added) {
							tuntun_tooltip_add_title (self);
							title_added = TRUE;
						}
						tuntun_tooltip_add_connection (self, connection, TUNTUN_CONSTANTS_IMAGES_CONNECTION_STATUS_DISCONNECT);
					}
					(connection == NULL ? NULL : (connection = (g_object_unref (connection), NULL)));
				}
			}
		}
		gtk_widget_show_all (GTK_WIDGET (self));
	}
	return obj;
}


static void tuntun_tooltip_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	TuntunTooltip * self;
	self = TUNTUN_TOOLTIP (object);
	switch (property_id) {
		case TUNTUN_TOOLTIP_CONNECTIONS:
		g_value_set_object (value, tuntun_tooltip_get_connections (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void tuntun_tooltip_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	TuntunTooltip * self;
	self = TUNTUN_TOOLTIP (object);
	switch (property_id) {
		case TUNTUN_TOOLTIP_CONNECTIONS:
		tuntun_tooltip_set_connections (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void tuntun_tooltip_class_init (TuntunTooltipClass * klass) {
	tuntun_tooltip_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (TuntunTooltipPrivate));
	G_OBJECT_CLASS (klass)->get_property = tuntun_tooltip_get_property;
	G_OBJECT_CLASS (klass)->set_property = tuntun_tooltip_set_property;
	G_OBJECT_CLASS (klass)->constructor = tuntun_tooltip_constructor;
	G_OBJECT_CLASS (klass)->dispose = tuntun_tooltip_dispose;
	g_object_class_install_property (G_OBJECT_CLASS (klass), TUNTUN_TOOLTIP_CONNECTIONS, g_param_spec_object ("connections", "connections", "connections", TUNTUN_TYPE_CONNECTIONS, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void tuntun_tooltip_instance_init (TuntunTooltip * self) {
	self->priv = TUNTUN_TOOLTIP_GET_PRIVATE (self);
}


static void tuntun_tooltip_dispose (GObject * obj) {
	TuntunTooltip * self;
	self = TUNTUN_TOOLTIP (obj);
	(self->priv->_connections == NULL ? NULL : (self->priv->_connections = (g_object_unref (self->priv->_connections), NULL)));
	G_OBJECT_CLASS (tuntun_tooltip_parent_class)->dispose (obj);
}


GType tuntun_tooltip_get_type (void) {
	static volatile gsize tuntun_tooltip_type_id = 0;
	if (g_once_init_enter (&tuntun_tooltip_type_id)) {
		GType tuntun_tooltip_type_id_temp;
		static const GTypeInfo g_define_type_info = { sizeof (TuntunTooltipClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) tuntun_tooltip_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (TuntunTooltip), 0, (GInstanceInitFunc) tuntun_tooltip_instance_init };
		tuntun_tooltip_type_id_temp = g_type_register_static (GTK_TYPE_VBOX, "TuntunTooltip", &g_define_type_info, 0);
		g_once_init_leave (&tuntun_tooltip_type_id, tuntun_tooltip_type_id_temp);
	}
	return tuntun_tooltip_type_id;
}




