/*
 *  tuntun-utils.vala is a part of Tuntun
 *
 *  Tuntun: a simple applet to manage OpenVPN connections
 *
 *  Copyright (C) 2008  Andrea Del Signore
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
 *
 *  Author:
 *     Andrea Del Signore <sejerpz@tin.it>       
 */

#include "tuntun-utils.h"
#include <config.h>




enum  {
	TUNTUN_UTILS_DUMMY_PROPERTY
};
static GtkBuilder* tuntun_utils_ui = NULL;
static void tuntun_utils_initialize_gtk_builder (void);
static gpointer tuntun_utils_parent_class = NULL;

static const char* TUNTUN_UTILS_GNOME_DOT_GNOME = ".gnome2";


/*
 * This function is the equivalent of
 * gnome_util_home_file contained in
 * libgnome-2.0/gnome-util.h
 */
char* tuntun_utils_gnome_util_home_file (const char* afile) {
	g_return_val_if_fail (afile != NULL, NULL);
	return g_build_filename (g_get_home_dir (), TUNTUN_UTILS_GNOME_DOT_GNOME, afile, NULL);
}


char* tuntun_utils_get_image_path (const char* id) {
	g_return_val_if_fail (id != NULL, NULL);
	return g_build_filename (PACKAGE_DATA_DIR, "tuntun", "pixmaps", id, NULL);
}


char* tuntun_utils_get_ui_path (const char* id) {
	g_return_val_if_fail (id != NULL, NULL);
	return g_build_filename (PACKAGE_DATA_DIR, "tuntun", "ui", id, NULL);
}


GtkBuilder* tuntun_utils_get_ui (void) {
	if (tuntun_utils_ui == NULL) {
		tuntun_utils_initialize_gtk_builder ();
	}
	return tuntun_utils_ui;
}


static void tuntun_utils_initialize_gtk_builder (void) {
	GError * inner_error;
	inner_error = NULL;
	{
		GtkBuilder* _tmp0;
		char* _tmp1;
		_tmp0 = NULL;
		tuntun_utils_ui = (_tmp0 = gtk_builder_new (), (tuntun_utils_ui == NULL ? NULL : (tuntun_utils_ui = (g_object_unref (tuntun_utils_ui), NULL))), _tmp0);
		gtk_builder_set_translation_domain (tuntun_utils_ui, GETTEXT_PACKAGE);
		_tmp1 = NULL;
		gtk_builder_add_from_file (tuntun_utils_ui, (_tmp1 = tuntun_utils_get_ui_path ("tuntun.ui")), &inner_error);
		if (inner_error != NULL) {
			goto __catch6_g_error;
		}
		_tmp1 = (g_free (_tmp1), NULL);
	}
	goto __finally6;
	__catch6_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			tuntun_utils_display_error ("initialize_gtk_builder", err->message);
			(err == NULL ? NULL : (err = (g_error_free (err), NULL)));
		}
	}
	__finally6:
	;
}


void tuntun_utils_display_error (const char* function, const char* message) {
	GtkMessageDialog* dialog;
	char* _tmp0;
	g_return_if_fail (function != NULL);
	g_return_if_fail (message != NULL);
	dialog = g_object_ref_sink (((GtkMessageDialog*) (gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, message, NULL))));
	_tmp0 = NULL;
	g_object_set (dialog, "secondary-text", function, NULL);
	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_object_destroy (GTK_OBJECT (dialog));
	(dialog == NULL ? NULL : (dialog = (g_object_unref (dialog), NULL)));
}


TuntunUtils* tuntun_utils_new (void) {
	TuntunUtils* self;
	self = ((TuntunUtils*) (g_type_create_instance (TUNTUN_TYPE_UTILS)));
	return self;
}


static void tuntun_utils_class_init (TuntunUtilsClass * klass) {
	tuntun_utils_parent_class = g_type_class_peek_parent (klass);
}


static void tuntun_utils_instance_init (TuntunUtils * self) {
	self->ref_count = 1;
}


GType tuntun_utils_get_type (void) {
	static volatile gsize tuntun_utils_type_id = 0;
	if (g_once_init_enter (&tuntun_utils_type_id)) {
		GType tuntun_utils_type_id_temp;
		static const GTypeInfo g_define_type_info = { sizeof (TuntunUtilsClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) tuntun_utils_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (TuntunUtils), 0, (GInstanceInitFunc) tuntun_utils_instance_init };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		tuntun_utils_type_id_temp = g_type_register_fundamental (g_type_fundamental_next (), "TuntunUtils", &g_define_type_info, &g_define_type_fundamental_info, 0);
		g_once_init_leave (&tuntun_utils_type_id, tuntun_utils_type_id_temp);
	}
	return tuntun_utils_type_id;
}


gpointer tuntun_utils_ref (gpointer instance) {
	TuntunUtils* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void tuntun_utils_unref (gpointer instance) {
	TuntunUtils* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		g_type_free_instance (((GTypeInstance *) (self)));
	}
}




